@@ -7,7 +7,7 @@
  *
  * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
  *
- * @(#)$Id: edit.h,v 1.1.1.1 1999/07/16 21:21:13 toast Exp $
+ * @(#)$Id: edit.h,v 1.2 2001/12/09 05:01:55 toast Exp $
  */
 
 #ifndef _EDIT_H_
@@ -21,7 +21,7 @@ extern	void	send_text();
 extern	void	eval_inputlist();
 extern	void	parse_command();
 extern	void	parse_line();
-extern	void	edit_char _((unsigned char));
+extern	void	edit_char _((u_char));
 extern	void	ExecuteTimers();
 extern	void	ison_ison();
 extern	void	query _((char *, char *, char *));
