@@ -17,6 +17,10 @@ extern	int	num_format_exists(int);
 enum FMT_TYPES {
 	ACTION_FMT,
 	ACTION_OTHER_FMT,
+	CTCP_FMT,
+	CTCP_PRIVATE_FMT,
+	CTCP_REPLY_FMT,
+	CTCP_UNKNOWN_FMT,
 	DCC_CHAT_FMT,
 	DCC_CHAT_ACTION_FMT,
 	DCC_TALK_FMT,
@@ -35,6 +39,8 @@ enum FMT_TYPES {
 	NOTIFY_SIGNOFF_FMT,
 	NOTIFY_SIGNON_FMT,
 	NOTIFY_SIGNON_USERHOST_FMT,
+	OPS_FMT,
+	OPS_OTHER_FMT,
 	PART_FMT,
 	PART_SELF_FMT,
 	PUBLIC_FMT,
@@ -51,6 +57,8 @@ enum FMT_TYPES {
 	SEND_MSG_FMT,
 	SEND_NOTICE_FMT,
 	SEND_OPERWALL_FMT,
+	SEND_OPS_FMT,
+	SEND_OPS_OTHER_FMT,
 	SEND_PUBLIC_FMT,
 	SEND_PUBLIC_ACTION_FMT,
 	SEND_PUBLIC_MSG_FMT,
@@ -59,10 +67,22 @@ enum FMT_TYPES {
 	SERVER_KILL_FMT,
 	SERVER_NOTICE_FMT,
 	SERVER_WALLOPS_FMT,
+	TOPIC_FMT,
 	QUIT_FMT,
 	UMODE_FMT,
 	WALL_FMT,
 	WALLOPS_FMT,
+	WHOIS_ADMIN_FMT,
+	WHOIS_AWAY_FMT,
+	WHOIS_CHANNELS_FMT,
+	WHOIS_IDLE_FMT,
+	WHOIS_IRCNAME_FMT,
+	WHOIS_NSLOOKUP_FMT,
+	WHOIS_OPER_FMT,
+	WHOIS_SERVER_FMT,
+	WHOIS_SIGNON_FMT,
+	WHOIS_USERHOST_FMT,
+	WHOWAS_USERHOST_FMT,
 	NUMBER_OF_FORMATS
 };
 
