@@ -5,7 +5,7 @@
  *
  * See the copyright file, or do a help ircii copyright 
  *
- * @(#)$Id: if.h,v 1.1.1.1 1999/07/16 21:21:13 toast Exp $
+ * @(#)$Id: if.h,v 1.2 2001/11/30 00:00:48 toast Exp $
  */
 
 #ifndef __if_h
@@ -19,5 +19,7 @@ extern	void	foreach _((char *, char *, char *));
 extern	void	fe _((char *, char *, char *));
 extern	void	forcmd _((char *, char *, char *));
 extern	void	fec _((char *, char *, char *));
+extern	void	repeatcmd _((char *, char *, char *));
+extern	void	switchcmd _((char *, char *, char *));
 
 #endif /* __if_h */
