@@ -7,7 +7,7 @@
  *
  * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
  *
- * @(#)$Id: ircaux.h,v 1.1.1.1 1999/07/16 21:21:13 toast Exp $
+ * @(#)$Id: ircaux.h,v 1.4 2001/12/02 18:23:00 toast Exp $
  */
 
 #ifndef _IRCAUX_H_
@@ -28,6 +28,7 @@ extern	char	*double_quote();
 extern	char	*new_malloc();
 extern	char	*new_realloc();
 extern	void	malloc_strcpy();
+extern	char	*m_strdup();
 extern	void	malloc_strcat();
 extern	void	new_free();
 extern	void	wait_new_free();
@@ -35,9 +36,13 @@ extern	FILE	*zcat();
 extern	int	is_number();
 extern	int	my_stricmp _((char *, char *));
 extern	int	my_strnicmp _((char *, char *, int));
+extern	char	*my_strstr _((char *, char *));
 extern	int	scanstr _((char *, char *));
+extern	int	counstr _((char *, char *));
 extern	void	really_free();
 extern	void	strmcpy();
 extern	void	strmcat();
+extern	char	*tdiff _((time_t));
+extern	char	*tdiff2 _((time_t));
 
 #endif /* _IRCAUX_H_ */
