@@ -1,15 +1,15 @@
 /*
  * alias.c Handles command aliases for irc.c 
  *
- * Written By Michael Sandrof
+ * Written By Timothy Jensen
  *
- * Copyright(c) 1990 
+ * Copyright (c) 1999-2001 
  *
- * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
+ * See the COPYRIGHT file, or do a /HELP COPYRIGHT 
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: alias.c,v 1.11 1999/07/25 22:55:12 toast Exp $";
+static	char	rcsid[] = "@(#)$Id: alias.c,v 1.29 2001/12/09 04:29:22 toast Exp $";
 #endif
 
 #include "irc.h"
@@ -127,6 +127,8 @@ static	BuiltIns built_in[] =
 	char	*function_left _((char *));
 	char	*function_right _((char *));
 	char	*function_mid _((char *));
+	char	*function_before _((char *));
+	char	*function_after _((char *));
 	char	*function_rand _((char *));
 	char	*function_srand _((char *));
 	char	*function_time _((char *));
@@ -170,7 +172,11 @@ static	BuiltIns built_in[] =
 	char	*function_lformat _((char *));
 	char	*function_center _((char *));
 	char	*function_sandr _((char *));
+	char	*function_tr _((char *));
 	char	*function_notword _((char *));
+	char	*function_strcnt _((char *));
+	char	*function_shift _((char *));
+	char	*function_unshift _((char *));
 	char	*function_pop _((char *));
 	char	*function_push _((char *));
 	char	*function_pluck _((char *));
@@ -180,6 +186,12 @@ static	BuiltIns built_in[] =
 	char	*function_isort _((char *));
 	char	*function_irsort _((char *));
 	char	*function_jot _((char *));
+	char	*function_isdigit _((char *));
+	char	*function_isalpha _((char *));
+	char	*function_chr _((char *));
+	char	*function_ascii _((char *));
+	char	*function_idle _((char *));
+	char	*function_uptime _((char *));
 
 typedef struct
 {
@@ -192,6 +204,8 @@ static BuiltInFunctions	built_in_functions[] =
 	{ "LEFT",		function_left },
 	{ "RIGHT",		function_right },
 	{ "MID",		function_mid },
+	{ "BEFORE",		function_before },
+	{ "AFTER",		function_after },
 	{ "RAND",		function_rand },
 	{ "SRAND",		function_srand },
 	{ "TIME",		function_time },
@@ -238,7 +252,12 @@ static BuiltInFunctions	built_in_functions[] =
 	{ "LFORMAT",		function_lformat },
 	{ "CENTER",		function_center },
 	{ "SANDR",		function_sandr },
+	{ "SAR",		function_sandr },
+	{ "TR",			function_tr },
 	{ "NOTWORD",		function_notword },
+	{ "STRCNT",		function_strcnt },
+	{ "SHIFT",		function_shift },
+	{ "UNSHIFT",		function_unshift },
 	{ "POP",		function_pop },
 	{ "PUSH",		function_push },
 	{ "PLUCK",		function_pluck },
@@ -260,6 +279,13 @@ static BuiltInFunctions	built_in_functions[] =
 	{ "FREAD",		function_fread },
 	{ "FREADLN",		function_freadln },
 	{ "JOT",		function_jot },
+
+	{ "ISDIGIT",		function_isdigit },
+	{ "ISALPHA",		function_isalpha },
+	{ "CHR",		function_chr },
+	{ "ASCII",		function_ascii },
+	{ "IDLE",		function_idle },
+	{ "UPTIME",		function_uptime },
 	{ (char *) 0,		NULL }
 };
 
@@ -492,7 +518,7 @@ built_in_alias(c)
 	BuiltIns	*tmp;
 	char	*ret = (char *) 0;
 
-	for(tmp = built_in;tmp->name;tmp++)
+	for(tmp = built_in;(tmp && tmp->name);tmp++)
 		if (c == tmp->name)
 		{
 			malloc_strcpy(&ret, tmp->func());
@@ -2321,6 +2347,121 @@ function_mid(input)
 	return result;
 }
 
+char	*
+function_before(input)
+	char	*input;
+{
+	char	*result = NULL;
+	char	*find,
+		*index;
+	int	which,
+		count;
+
+	malloc_strcpy(&result, empty_string);
+
+	find = next_arg(input, &input);
+	if (find && *find)
+	{
+		which = atoi(find);	
+		if (which)
+		{
+			find = next_arg(input, &input);
+			if (!find || !*find)
+			{
+				return result;
+			}
+			if (which < 0)
+			{
+				count = countstr(input, find);
+				which = count + which + 1;
+				if (which < 1)
+				{
+					return result;
+				}
+			}
+
+		}
+		else
+		{
+			which = 1;
+		}
+		index = input;
+		while (which > 0)
+		{
+			index = my_strstr(index , find);
+			if (index)
+			{
+				if (which-- == 1)
+				{
+					*index = 0;
+				}
+				index += strlen(find);
+			}
+			else
+			{
+				return result;
+			}
+		}
+		malloc_strcpy(&result, input);
+	}
+	return result;
+}
+
+char	*
+function_after(input)
+	char	*input;
+{
+	char	*result = NULL;
+	char	*find;
+	int	which,
+		count;
+
+	malloc_strcpy(&result, empty_string);
+
+	find = next_arg(input, &input);
+	if (find && *find)
+	{
+		which = atoi(find);
+		if (which)
+		{
+			find = next_arg(input, &input);
+			if (!find || !*find)
+			{
+				return result;
+			}
+			if (which < 0)
+			{
+				count = countstr(input, find);
+				which = count + which + 1;
+				if (which < 1)
+				{
+					return result;
+				}
+			}
+
+		}
+		else
+		{
+			which = 1;
+		}
+		while (which > 0)
+		{
+			input = my_strstr(input, find);
+			if (input)
+			{
+				input += strlen(find);
+				which--;
+			}
+			else
+			{
+				return result;
+			}
+		}
+		malloc_strcpy(&result, input);
+	}
+	return result;
+}
+
 /* patch from Sarayan to make $rand() better */
 
 #define RAND_A 16807L
@@ -2390,11 +2531,10 @@ function_time(input)
 {
 	char	*result = (char *) 0;
 	time_t	ltime;
-	char	tmp[40];
 
 	(void) time(&ltime);
-	sprintf(tmp, "%ld", ltime);
-	malloc_strcpy(&result, tmp);
+	sprintf(buffer, "%ld", ltime);
+	malloc_strcpy(&result, buffer);
 	return result;
 }
 
@@ -2405,9 +2545,16 @@ function_stime(input)
 	char	*result = (char *) 0;
 	time_t	ltime;
 
-	ltime = atol(input);
-	malloc_strcpy(&result, ctime(&ltime));
-	result[strlen(result) - 1] = (char) 0;
+	if (input && *input)
+	{
+		ltime = atol(input);
+		malloc_strcpy(&result, ctime(&ltime));
+		result[strlen(result) - 1] = (char) 0;
+	}
+	else
+	{
+		malloc_strcpy(&result, empty_string);
+	}
 	return result;
 }
 
@@ -2416,43 +2563,32 @@ function_tdiff(input)
 	char	*input;
 {
 	char	*result = (char *) 0;
-	time_t	ltime;
-	time_t	days,
-		hours,
-		minutes,
-		seconds;
-	char	tmp[80];
-	char	*tstr;
 
-	ltime = atol(input);
-	seconds = ltime % 60;
-	ltime = (ltime - seconds) / 60;
-	minutes = ltime%60;
-	ltime = (ltime - minutes) / 60;
-	hours = ltime % 24;
-	days = (ltime - hours) / 24;
-	tstr = tmp;
-	if (days)
+	if (input && *input)
 	{
-		sprintf(tstr, "%d day%s ", days, (days==1)?"":"s");
-		tstr += strlen(tstr);
+		malloc_strcpy(&result, tdiff(atoi(input)));
 	}
-	if (hours)
+	else
 	{
-		sprintf(tstr, "%d hour%s ", hours, (hours==1)?"":"s");
-		tstr += strlen(tstr);
+		malloc_strcpy(&result, empty_string);
 	}
-	if (minutes)
+	return result;
+}
+
+char  *
+function_tdiff2(input)
+	char    *input;
+{
+	char    *result = (char *) 0;
+
+	if (input && *input)
 	{
-		sprintf(tstr, "%d minute%s ", minutes, (minutes==1)?"":"s");
-		tstr += strlen(tstr);
+		malloc_strcpy(&result, tdiff2(atoi(input)));
 	}
-	if (seconds || (!days && !hours && !minutes))
+	else
 	{
-		sprintf(tstr, "%d second%s", seconds, (seconds==1)?"":"s");
-		tstr += strlen(tstr);
+		malloc_strcpy(&result, empty_string);
 	}
-	malloc_strcpy(&result, tmp);
 	return result;
 }
 
@@ -2464,13 +2600,19 @@ function_index(input)
 	char	*schars;
 	char	*iloc;
 	int	ival;
-	char	tmp[40];
 
-	schars = next_arg(input, &input);
-	iloc = (schars) ? sindex(input, schars) : NULL;
-	ival = (iloc) ? iloc - input : -1;
-	sprintf(tmp, "%d", ival);
-	malloc_strcpy(&result, tmp);
+	if (input && *input)
+	{
+		schars = next_arg(input, &input);
+		iloc = (schars) ? sindex(input, schars) : NULL;
+		ival = (iloc) ? iloc - input : -1;
+		sprintf(buffer, "%d", ival);
+		malloc_strcpy(&result, buffer);
+	}
+	else
+	{
+		malloc_strcpy(&result, empty_string);
+	}
 	return result;
 }
 
@@ -2482,19 +2624,25 @@ function_rindex(input)
 	char	*schars;
 	char	*iloc, *liloc;
 	int	ival;
-	char	tmp[40];
 
-	schars = next_arg(input, &input);
-	iloc = NULL;
-	if (schars)
+	if (input && *input)
 	{
-		liloc = sindex(input, schars);
-		while (liloc)
-			liloc = sindex((iloc = liloc) + 1, schars);
+		schars = next_arg(input, &input);
+		iloc = NULL;
+		if (schars)
+		{
+			liloc = sindex(input, schars);
+			while (liloc)
+				liloc = sindex((iloc = liloc) + 1, schars);
+		}
+		ival = (iloc) ? iloc-input : -1;
+		sprintf(buffer, "%d", ival);
+		malloc_strcpy(&result, buffer);
+	}
+	else
+	{
+		malloc_strcpy(&result, empty_string);
 	}
-	ival = (iloc) ? iloc-input : -1;
-	sprintf(tmp, "%d", ival);
-	malloc_strcpy(&result, tmp);
 	return result;
 }
 
@@ -2509,23 +2657,29 @@ function_match(input)
 	int	best_match = 0;
 	int	match = 0;
 	int	match_index = 0;
-	char	tmp[40];
 
-	if ((pattern = next_arg(input, &input)) != NULL)
+	if (input && *input)
 	{
-		while ((word = next_arg(input, &input)) != NULL)
+		if ((pattern = next_arg(input, &input)) != NULL)
 		{
-			match_index++;
-			if ((current_match = wild_match(pattern, word))
-					> best_match)
+			while ((word = next_arg(input, &input)) != NULL)
 			{
-				match = match_index;
-				best_match = current_match;
+				match_index++;
+				if ((current_match = wild_match(pattern, word))
+					> best_match)
+				{
+					match = match_index;
+					best_match = current_match;
+				}
 			}
 		}
+		sprintf(buffer, "%d", match);
+		malloc_strcpy(&result, buffer);
+	}
+	else
+	{
+		malloc_strcpy(&result, empty_string);
 	}
-	sprintf(tmp, "%d", match);
-	malloc_strcpy(&result, tmp);
 	return result;
 }
 
@@ -2540,23 +2694,29 @@ function_rmatch(input)
 	int	best_match = 0;
 	int	match = 0;
 	int	rmatch_index = 0;
-	char	tmp[40];
 
-	if ((pattern = next_arg(input, &input)) != NULL)
+	if (input && *input)
 	{
-		while ((word = next_arg(input, &input)) != NULL)
+		if ((pattern = next_arg(input, &input)) != NULL)
 		{
-			rmatch_index++;
-			if ((current_match = wild_match(word, pattern)) >
-					best_match)
+			while ((word = next_arg(input, &input)) != NULL)
 			{
-				match = rmatch_index;
-				best_match = current_match;
+				rmatch_index++;
+				if ((current_match =
+					wild_match(word, pattern)) > best_match)
+				{
+					match = rmatch_index;
+					best_match = current_match;
+				}
 			}
 		}
+		sprintf(buffer, "%d", match);
+		malloc_strcpy(&result, buffer);
+	}
+	else
+	{
+		malloc_strcpy(&result, empty_string);
 	}
-	sprintf(tmp, "%d", match);
-	malloc_strcpy(&result, tmp);
 	return result;
 }
 
@@ -2580,16 +2740,23 @@ function_strip(input)
 	char	*chars;
 	char	*cp, *dp;
 
-	if ((chars = next_arg(input, &input)) && input)
+	if (input && *input)
 	{
-		for (cp = input, dp = result; *cp; cp++)
+		if ((chars = next_arg(input, &input)) && input)
 		{
-			if (!index(chars, *cp))
-				*dp++ = *cp;
+			for (cp = input, dp = result; *cp; cp++)
+			{
+				if (!index(chars, *cp))
+					*dp++ = *cp;
+			}
+			*dp = '\0';
 		}
-		*dp = '\0';
+		malloc_strcpy(&retval, result);
+	}
+	else
+	{
+		malloc_strcpy(&retval, empty_string);
 	}
-	malloc_strcpy(&retval, result);
 	return retval;
 }
 
@@ -2693,13 +2860,12 @@ function_winnum(input)
 	char	*input;
 {
 	char	*result = (char *) 0;
-	char	tmp[10];
 
 	if (curr_scr_win)
-		sprintf(tmp, "%d", curr_scr_win->refnum);
+		sprintf(buffer, "%d", curr_scr_win->refnum);
 	else
-		strcpy(tmp, "-1");
-	malloc_strcpy(&result, tmp);
+		strcpy(buffer, "-1");
+	malloc_strcpy(&result, buffer);
 	return result;
 }
 
@@ -2980,13 +3146,12 @@ char *function_utime(input)
 char *input;
 {
 	char    *result = (char *) 0;
-	char    tmp[200];
 	struct  timeval         tp;
 	struct  timezone        tzp;
 	if (gettimeofday(&tp,&tzp) == 0)
 	{
-		sprintf(tmp,"%ld %ld",tp.tv_sec,tp.tv_usec);
-		malloc_strcpy(&result, tmp);
+		sprintf(buffer, "%ld %ld", tp.tv_sec, tp.tv_usec);
+		malloc_strcpy(&result, buffer);
 		return result;
 	}
 	else
@@ -3031,50 +3196,6 @@ char	*input;
 	return result;
 }
 
-char  *function_tdiff2(input)
-char    *input;
-{
-	char    *result = (char *) 0;
-	time_t  ltime;
-	time_t  days,
-		hours,
-		minutes,
-		seconds;
-	char    tmp[80];
-	char    *tstr;
-
-	ltime = atol(input);
-	seconds = ltime % 60;
-	ltime = (ltime - seconds) / 60;
-	minutes = ltime%60;
-	ltime = (ltime - minutes) / 60;
-	hours = ltime % 24;
-	days = (ltime - hours) / 24;
-	tstr = tmp;
-	if (days)
-	{
-		sprintf(tstr, "%dd ", days);
-		tstr += strlen(tstr);
-	}
-	if (hours)
-	{
-		sprintf(tstr, "%dh ", hours);
-		tstr += strlen(tstr);
-	}
-	if (minutes)
-	{
-		sprintf(tstr, "%dm ", minutes);
-		tstr += strlen(tstr);
-	}
-	if (seconds || (!days && !hours && !minutes))
-	{
-		sprintf(tstr, "%ds", seconds);
-		tstr += strlen(tstr);
-	}
-	malloc_strcpy(&result, tmp);
-	return result;
-}
-
 char	*
 function_clusterhost(input)
 	char	*input;
@@ -3101,11 +3222,10 @@ function_length(input)
 {
 	char	*result = (char *) 0;
 	int	lval;
-	char	tmp[40];
 
 	lval = strlen(input);
-	sprintf(tmp, "%d", lval);
-	malloc_strcpy(&result, tmp);
+	sprintf(buffer, "%d", lval);
+	malloc_strcpy(&result, buffer);
 	return result;
 }
 
@@ -3186,7 +3306,6 @@ function_lformat(input)
 		memset(result, 32, w);
 
 		strcpy(result, input);
-			malloc_strcpy(&result, empty_string);
 		result[l] = ' ';
 		result[w] = '\000';
 	} else
@@ -3233,43 +3352,188 @@ function_sandr(input)
 	char	*input;
 {
 	char	*result = (char *) 0;
-	char	*src, *rep, *words;
+	Alias	*alias = NULL;
+	char	*flags, *src, *rep, *words, *varname;
 	int	srclen;
 	int	wlen, wpos;
+	int	gflag = 0,
+		rflag = 0;
 	char	tmp[2];
 
 	tmp[1] = '\000';
 
-	if (input) {
-		src = input;
-		rep = index(input, '/');
-		if (rep) {
-			*rep++ = '\000';
-			words = index(rep, '/');
-			if (words) {
-				*words++ = '\000';
-				if (!*src) {
-					malloc_strcpy(&result, words);
-					return result;
-				}
-				wlen = strlen(words);
-				srclen = strlen(src);
-				for (wpos = 0; wpos < wlen; wpos++)
-					if (!my_strnicmp(src, &words[wpos], srclen)) {
-						malloc_strcat(&result, rep);
-						if (srclen)
-							wpos += srclen-1;
-					} else {
-						tmp[0] = words[wpos];
-						malloc_strcat(&result, tmp); 
+	if (input)
+	{
+		flags = input;
+		src = index(flags, '/');
+		if (src)
+		{
+			*src++ = '\000';
+			rep = index(src, '/');
+			if (rep)
+			{
+				*rep++ = '\000';
+				words = index(rep, '/');
+				if (words)
+				{
+					*words++ = '\000';
+					gflag = index(flags, 'g') ||
+						index(flags, 'G');
+					rflag = index(flags, 'r') ||
+						index(flags, 'R');
+					if (rflag)
+					{
+						varname = next_arg(words, NULL);
+						alias = find_alias(&(alias_list[VAR_ALIAS]), varname, 0, (int *) NULL);
+						if (!alias)
+						{
+							alias = (Alias *)new_malloc(sizeof(Alias));
+							if (!alias)
+							{
+								malloc_strcpy(&result, empty_string);
+								return result;
+							}
+							alias->name = (char *) 0;
+							alias->stuff = (char *) 0;
+							malloc_strcpy(&alias->name, upper(varname));
+							malloc_strcpy(&alias->stuff, empty_string);
+							insert_alias(&(alias_list[VAR_ALIAS]), alias);
+						}
+						words = alias->stuff;
 					}
-			} else
+					if (!*src)
+					{
+						malloc_strcpy(&result, words);
+						return result;
+					}
+					wlen = strlen(words);
+					srclen = strlen(src);
+					for (wpos = 0; wpos < wlen; wpos++)
+					{
+						if (!my_strnicmp(src, &words[wpos], srclen))
+						{
+							malloc_strcat(&result, rep);
+							if (srclen)
+							{
+								wpos += srclen-1;
+							}
+							if (!gflag)
+							{
+								malloc_strcat(&result, &words[wpos + 1]);
+								break;
+							}
+						}
+						else
+						{
+							tmp[0] = words[wpos];
+							malloc_strcat(&result, tmp); 
+						}
+					}
+					if (rflag)
+					{
+						malloc_strcpy(&(alias->stuff),
+							result);
+					}
+				}
+				else
+				{
+					malloc_strcpy(&result, empty_string);
+				}
+			}
+			else
+			{
 				malloc_strcpy(&result, empty_string);
-		} else
+			}
+		}
+		else
+		{
 			malloc_strcpy(&result, empty_string);
-	} else
+		}
+	}
+	else
+	{
 		malloc_strcpy(&result, empty_string);
+	}
+	return result;
+}
 
+char	*
+function_tr(input)
+	char	*input;
+{
+	char	*result = (char *) 0;
+	char	*flags,
+		*src,
+		*dst,
+		*target;
+	int	srcl, dstl;
+	int	i, j, k, l;
+	int	m;
+
+	if (input && *input)
+	{
+		flags = input;
+		src = index(input, '/');
+		if (src)
+		{
+			*(src++) = 0;
+			dst = index(src, '/');
+			if (dst)
+			{
+				*(dst++) = 0;
+				target = index(dst, '/');
+				if (target)
+				{
+					*(target++) = 0;
+					srcl = strlen(src);
+					dstl = strlen(dst);
+					malloc_strcpy(&result, target);
+					l = 0;
+					for (i = 0; target[i]; i++)
+					{
+						m = 0;
+						for (j = 0, k = 0; j < srcl; j++)
+						{
+							if (target[i] == src[j])
+							{
+								if (dst[k])
+								{
+									result[l++] = dst[k];
+								}
+								m = 1;
+								break;
+							}
+							if (k < (dstl - 1))
+							{
+								k++;
+							}
+						}
+						if (!m)
+						{
+							result[l++] = target[i];
+						}
+					}
+					result[l] = 0;
+				}
+				else
+				{
+					malloc_strcpy(&result, empty_string);
+				}
+			}
+			else
+			{
+				malloc_strcpy(&result, empty_string);
+			}
+		}
+		else
+		{
+			malloc_strcpy(&result, empty_string);
+		}
+	}
+	else
+	{
+		malloc_strcpy(&result, empty_string);
+	}
 	return result;
 }
 
@@ -3302,13 +3566,31 @@ function_notword(input)
 }
 
 char	*
-function_pop(input)
+function_strcnt(input)
+	char	*input;
+{
+	char	*result = (char *) 0;
+	int	count = 0;
+
+	if (input)
+	{
+		while (next_arg(input, &input))
+		{
+			count++;
+		}
+	}
+	sprintf(buffer, "%d", count);
+	malloc_strcpy(&result, buffer);
+	return result;
+}
+
+char	*
+function_shift(input)
 	char	*input;
 {
 	char	*result = (char *) 0;
 	Alias	*alias;
-	char	*list, *aword;
-	char	*words = (char *) 0;
+	char	*list, *space;
 
 	if (input) {
 		list = next_arg(input, &input);
@@ -3318,12 +3600,104 @@ function_pop(input)
 			if ((alias = find_alias(&(alias_list[VAR_ALIAS]), list, 0, (int *) NULL))
 					!= NULL)
 			{
-				malloc_strcpy(&words, alias->stuff);
-				list = words;
-				aword = next_arg(words, &words);
-				malloc_strcpy(&result, aword);
-				malloc_strcpy(&alias->stuff, words);
-				new_free(&list);
+				space = index(alias->stuff, ' ');
+				if (space)
+				{
+					*(space++) = 0;
+					malloc_strcpy(&result, alias->stuff);
+					malloc_strcpy(&alias->stuff, space);
+				}
+				else
+				{
+					malloc_strcpy(&result, alias->stuff);
+					malloc_strcpy(&alias->stuff,
+						empty_string);
+				}
+			} else
+				malloc_strcpy(&result, empty_string);
+		} else
+			malloc_strcpy(&result, empty_string);
+	} else
+		malloc_strcpy(&result, empty_string);
+
+	return result;
+}
+
+char	*
+function_unshift(input)
+	char	*input;
+{
+	char	*result = (char *) 0;
+	char	*tmp = (char *) 0;
+	Alias	*alias;
+	char	*list;
+
+	if (input) {
+		list = next_arg(input, &input);
+		if (list) {
+			if (*list == '$')
+				list++;
+			if (input && *input)
+				if ((alias = find_alias(&(alias_list[VAR_ALIAS]), list, 0, (int *) NULL))
+						!= NULL)
+				{
+					malloc_strcpy(&tmp, input);
+					if (*(alias->stuff))
+					{
+						malloc_strcat(&tmp, " ");
+						malloc_strcat(&tmp,
+							alias->stuff);
+					}
+					malloc_strcpy(&alias->stuff, tmp);
+					new_free(&tmp);
+					malloc_strcpy(&result, alias->stuff);
+				} else {
+					alias = (Alias *)new_malloc(sizeof(Alias));
+					alias->name = (char *) 0;
+					alias->stuff = (char *) 0;
+					malloc_strcpy(&alias->name, upper(list));
+					malloc_strcpy(&alias->stuff, input);
+					insert_alias(&(alias_list[VAR_ALIAS]), alias);
+					malloc_strcpy(&result, alias->stuff);
+				}
+			else
+				malloc_strcpy(&result, empty_string);
+		} else
+			malloc_strcpy(&result, empty_string);
+	} else
+		malloc_strcpy(&result, empty_string);
+
+	return result;
+}
+
+char	*
+function_pop(input)
+	char	*input;
+{
+	char	*result = (char *) 0;
+	Alias	*alias;
+	char	*list, *space;
+
+	if (input) {
+		list = next_arg(input, &input);
+		if (list) {
+			if (*list == '$')
+				list++;
+			if ((alias = find_alias(&(alias_list[VAR_ALIAS]), list, 0, (int *) NULL))
+					!= NULL)
+			{
+				space = rindex(alias->stuff, ' ');
+				if (space)
+				{
+					*(space++) = 0;
+					malloc_strcpy(&result, space);
+				}
+				else
+				{
+					malloc_strcpy(&result, alias->stuff);
+					malloc_strcpy(&alias->stuff,
+						empty_string);
+				}
 			} else
 				malloc_strcpy(&result, empty_string);
 		} else
@@ -3340,7 +3714,7 @@ function_push(input)
 {
 	char	*result = (char *) 0;
 	Alias	*alias;
-	char	*list, *words = (char *) 0;
+	char	*list;
 
 	if (input) {
 		list = next_arg(input, &input);
@@ -3351,12 +3725,13 @@ function_push(input)
 				if ((alias = find_alias(&(alias_list[VAR_ALIAS]), list, 0, (int *) NULL))
 						!= NULL)
 				{
-					malloc_strcpy(&words, input);
-					malloc_strcat(&words, " ");
-					malloc_strcat(&words, alias->stuff);
-					malloc_strcpy(&alias->stuff, words);
-					new_free(&words);
-					malloc_strcpy(&result, "1");
+					if (*(alias->stuff))
+					{
+						malloc_strcat(&alias->stuff,
+							" ");
+					}
+					malloc_strcat(&alias->stuff, input);
+					malloc_strcpy(&result, alias->stuff);
 				} else {
 					alias = (Alias *)new_malloc(sizeof(Alias));
 					alias->name = (char *) 0;
@@ -3364,14 +3739,14 @@ function_push(input)
 					malloc_strcpy(&alias->name, upper(list));
 					malloc_strcpy(&alias->stuff, input);
 					insert_alias(&(alias_list[VAR_ALIAS]), alias);
-					malloc_strcpy(&result, "1");
+					malloc_strcpy(&result, alias->stuff);
 				}
 			else
-				malloc_strcpy(&result, "0");
+				malloc_strcpy(&result, empty_string);
 		} else
-			malloc_strcpy(&result, "0");
+			malloc_strcpy(&result, empty_string);
 	} else
-		malloc_strcpy(&result, "0");
+		malloc_strcpy(&result, empty_string);
 
 	return result;
 }
@@ -3635,11 +4010,10 @@ char	*
 function_jot(input)
 	char	*input;
 {
-	char	tmp[16];
 	char	*result = (char *) 0;
 	char	*fromstr, *tostr, *intstr;
-	long	from, to, interval;
-	long	i;
+	int	from, to, interval;
+	int	i;
 
 	malloc_strcpy(&result, empty_string);
 
@@ -3663,16 +4037,129 @@ function_jot(input)
 			for (i = from; i <= to; i += interval) {
 				if (*result)
 					malloc_strcat(&result, " ");
-				sprintf(tmp, "%d", i);
-				malloc_strcat(&result, tmp);
+				sprintf(buffer, "%d", i);
+				malloc_strcat(&result, buffer);
 			}
 		else
 			for (i = from; i >= to; i += interval) {
 				if (*result)
 					malloc_strcat(&result, " ");
-				sprintf(tmp, "%d", i);
-				malloc_strcat(&result, tmp);
+				sprintf(buffer, "%d", i);
+				malloc_strcat(&result, buffer);
 			}
 	}
 	return result;
 }
+
+char	*
+function_isdigit(input)
+	char	*input;
+{
+	char	*result = (char *) 0;
+
+	if (input)
+	{
+		if (isdigit(*input))
+		{
+			malloc_strcpy(&result, "1");
+		}
+		else
+		{
+			malloc_strcpy(&result, "0");
+		}
+	}
+	else
+	{
+		malloc_strcpy(&result, empty_string);
+	}
+	return result;
+}
+
+char	*
+function_isalpha(input)
+	char	*input;
+{
+	char	*result = (char *) 0;
+
+	if (input)
+	{
+		if (isalpha(*input))
+		{
+			malloc_strcpy(&result, "1");
+		}
+		else
+		{
+			malloc_strcpy(&result, "0");
+		}
+	}
+	else
+	{
+		malloc_strcpy(&result, empty_string);
+	}
+	return result;
+}
+
+char	*
+function_chr(input)
+	char	*input;
+{
+	char	*result = (char *) 0;
+	char	tmp[2] = { 0, 0 };
+	char	*item;
+
+	malloc_strcpy(&result, empty_string);
+	if (input && *input)
+	{
+		while (NULL != (item = next_arg(input, &input)))
+		{
+			tmp[0] = atoi(item);
+			malloc_strcat(&result, (char *)tmp);
+		}
+	}
+	return result;
+}
+
+char	*
+function_ascii(input)
+	char	*input;
+{
+	char	*result = (char *) 0;
+
+	malloc_strcpy(&result, empty_string);
+	if (input)
+	{
+		for (; *input; input++)
+		{
+			sprintf(buffer, "%d", *input);
+			if (*result)
+			{
+				malloc_strcat(&result, " ");
+			}
+			malloc_strcat(&result, buffer);
+		}
+	}
+	return result;
+}
+
+char	*
+function_idle(input)
+	char	*input;
+{
+	char	*result = (char *) 0;
+
+	sprintf(buffer, "%ld", time(0) - idle_time);
+	malloc_strcpy(&result, buffer);
+	return result;
+}
+
+char	*
+function_uptime(input)
+	char	*input;
+{
+	char	*result = (char *) 0;
+
+	sprintf(buffer, "%ld", time(0) - start_time);
+	malloc_strcpy(&result, buffer);
+	return result;
+}
+
