@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: comstud.c,v 1.10 1999/08/05 17:53:50 toast Exp $";
+static	char	rcsid[] = "@(#)$Id: comstud.c,v 1.29 2001/12/08 05:12:07 toast Exp $";
 #endif
 
 #include <stdio.h>
@@ -48,15 +48,19 @@ static	char	rcsid[] = "@(#)$Id: comstud.c,v 1.10 1999/08/05 17:53:50 toast Exp $
 #include "notify.h"
 #include "numbers.h"
 #include "status.h"
+#include "hook.h"
 #include "toast.h"
+#include "format.h"
+#include "comstud.h"
 
 #define getrandom(min, max) ((rand() % (int)(((max)+1) - (min))) + (min))
 
-/*
+
 #define CHECK_FOR_OPS	/* define this if you want to
 			make sure you are opped before
 			doing a mode */ 
 
+int  ops_method = OPS_METHOD_DEFAULT;
 int  com_action = 0;
 char com_arg[2048];
 char com_nickflood[NICKNAME_LEN+1];
@@ -73,7 +77,9 @@ char *get_token(char **src, char *chars);
 extern char MyHostName[80];
 extern struct   in_addr MyHostAddr;             /* The local machine address */
 
-int isme(char *nick)
+int
+isme(nick)
+	char	*nick;
 {
 	return !my_stricmp(nick, get_server_nickname(from_server));
 }
@@ -85,7 +91,12 @@ irc.HACKS.Arizona.EDU *** Notice -- Received KILL message for imanoste. From
  .bu.edu]!cs-pub.bu.edu (imanoste <- irc2.ece.uiuc.edu)
 */
 
-void split_kill(path, who, who_uh, server, dead, reason)
+/* Hybrid-6 local kill:
+:irc.blackened.com NOTICE ToastTEST :*** Notice -- Received KILL message for killtest. From Toast Path: plasma.powered!toast!Toast (testing)
+*/
+
+void
+split_kill(path, who, who_uh, server, dead, reason)
 char *path;
 char **who;
 char **who_uh;
@@ -93,6 +104,8 @@ char **server;
 char **dead;
 char **reason;
 {
+	static char uhbuf[128];
+	char *uname, *hname;
 	char *temp;
 
 	*who = *who_uh = *server = *reason = NULL;
@@ -129,14 +142,33 @@ char **reason;
 	temp = strrchr(temp, '!');
 	if (!temp)
 		return;
-	*temp = '\0';
+	*temp = '\0';		/* chop off the nick */
+
 	temp--;
 	while (*temp && (*temp != ' ') && (*temp != '!'))
 		temp--;
-	*temp = '\0';
-	*who_uh = temp+1;
-	if (!*who_uh || !**who_uh)
-		*who_uh = NULL;
+	if ((*temp == '!') && !strchr(temp, '.')) {
+		*temp = '\0';
+		uname = temp+1;
+		temp--;
+		while (*temp && (*temp != ' ') && (*temp != '!'))
+			temp--;
+		*temp = '\0';
+		hname = temp+1;
+	} else {
+		*temp = '\0';
+		uname = temp+1;
+		hname = NULL;
+	}
+
+	if (!uname || !*uname)
+		uname = NULL;
+	if (hname != NULL) {
+		sprintf(uhbuf, "%s@%s", uname, hname);
+		*who_uh = (char *)&uhbuf;
+	} else if (uname != NULL)
+		*who_uh = uname;
+
 	temp--;
 	while (*temp && (*temp != ' ') && (*temp != '!'))
 		temp--;
@@ -182,8 +214,12 @@ char *reason;
 	if (nick)
 	{
 		blah = SmallTS((time_t *)&t);
-		fprintf(whee, "%s%s%s by %s@%s on %s %s\n",blah?blah:"",
-			blah?"> ":"", nick, from, host, server, reason);
+		if (strchr(from, '!') || strchr(from, '@'))
+			fprintf(whee, "%s%s%s by %s on %s %s\n",blah?blah:"",
+				blah?"> ":"", nick, from, server, reason);
+		else
+			fprintf(whee, "%s%s%s by %s@%s on %s %s\n",blah?blah:"",
+				blah?"> ":"", nick, from, host, server, reason);
 	}
 	else if (from)
 		fprintf(whee, "KillLog started at %s\n", SmallTS((time_t *)&t));
@@ -192,6 +228,48 @@ char *reason;
 	fclose(whee);
 }
 
+void
+logaddnolog(nick)
+char	*nick;
+{
+	char *timestr;
+	time_t t;
+
+	t = time(0);
+	if (logptr) {
+		fprintf(logptr, "*** Stopped logging messages to/from %s <%s>\n",
+			nick, SmallTS(&t));
+	}
+}
+
+void
+logremovenolog(nick)
+char	*nick;
+{
+	char *timestr;
+	time_t t;
+
+	t = time(0);
+	if (logptr) {
+		fprintf(logptr, "*** Started logging messages to/from %s <%s>\n",
+			nick, SmallTS(&t));
+	}
+}
+
+void
+lognolog(nicks)
+char	*nicks;
+{
+	char *timestr;
+	time_t t;
+
+	t = time(0);
+	if (nicks && *nicks)
+		fprintf(logptr, "*** Not logging messages to/from: %s <%s>\n", nicks, SmallTS(&t));
+	else
+		fprintf(logptr, "*** All nicks are being logged <%s>\n", SmallTS(&t));
+}
+
 int  logmsg(rc, type, from, string, extra, flag)
 int  rc;
 char *type;
@@ -205,6 +283,7 @@ int  flag;
 	time_t t;
 	char filename[255];
 	mode_t savemask;
+	char *nologlist;
 
 	/*
 	 * Check the NO_LOG variable for 'from'. If it contains that nick,
@@ -254,6 +333,8 @@ int  flag;
 		}
 		(void)umask(savemask);
 		fprintf(logptr, "MsgLog started at %s\n", SmallTS(&t));
+		if (nologlist = get_string_var(NO_LOG_VAR))
+			fprintf(logptr, "*** Not logging messages to/from: %s\n", nologlist);
 		fflush(logptr);
         	if (string)
 			return logmsg(rc, type, from, string, extra, 0);
@@ -287,7 +368,8 @@ int flag;
 	logmsg(0, NULL, NULL, NULL, NULL, flag ? 1 : 2);
 }
 
-void not_on_a_channel()
+void
+not_on_a_channel(void)
 {
         say("D'oh! You're not on a channel!");
 }
@@ -321,9 +403,10 @@ char *chars;
   return string;
 }
 
-int  freadln(stream, lin)
-FILE *stream;
-char *lin;
+int
+freadln(stream, lin)
+	FILE	*stream;
+	char	*lin;
 {
         char *p;
 
@@ -375,7 +458,8 @@ char *get_reason()
    return temp ? temp : get_string_var(DEFAULT_REASON_VAR);
 }
 
-char *do_nslookup(host)
+char *
+do_nslookup(host)
 char *host;
 {
   struct hostent *temp;
@@ -625,8 +709,9 @@ char *hostname;
 	return result;
 }
 
-void netfinger(name)
-char *name;
+void
+netfinger(name)
+	char	*name;
 {
         char *host;
         char fname[100], temp[2], tmpstr[255];
@@ -805,20 +890,22 @@ char *arg;
 	}
 }
 
-void userhost_nsl(stuff, nick, args)
+void
+userhost_nsl(stuff, nick, args)
 WhoisStuff *stuff;
 char *nick;
 char *args;
 {
 	char *nsl;
+	char *lookup;
 
-	if (!stuff || !stuff->nick || !nick || my_stricmp(stuff->nick, nick))
-	{
-		say("No information for %s", nick);
-                return;
-        }
-	nsl = do_nslookup(stuff->host);
-	say("%s is %s", stuff->host, nsl ? nsl : "unknown.");
+	if (!stuff || stuff->not_on || !stuff->nick || !nick || my_stricmp(stuff->nick, nick))
+		lookup = nick;
+	else
+		lookup = stuff->host;
+
+	nsl = do_nslookup(lookup);
+	say("%s is %s", lookup, nsl ? nsl : "unknown.");
 }
 
 void userhost_unban(stuff, nick, args)
@@ -849,6 +936,10 @@ char *args;
 
         if (!stuff || !stuff->nick || !nick || my_stricmp(stuff->nick, nick))
                 return;
+	if (stuff->not_on) {
+		say("%s is not on IRC!", nick);
+		return;
+	}
 	temp = stuff->user;
         if (temp && (*temp == '~'))
                 temp++;
@@ -872,6 +963,10 @@ char *args;
 
         if (!stuff || !stuff->nick || !nick || my_stricmp(stuff->nick, nick))
                 return;
+	if (stuff->not_on) {
+		say("%s is not on IRC!", nick);
+		return;
+	}
         sprintf(ignore_buf, "*@%s", cluster(stuff->host));
         if (args)
         {
@@ -961,8 +1056,8 @@ char *args;
 		temp++;
 	sprintf(uh, "*!*%s@%s", temp, cluster(stuff->host));
 	if (temptime > 0) {
-		say("Ban on %s will be removed in %d minute%s.",
-			uh, temptime, (temptime == 1) ? "" : "s");
+		say("Ban on %s will be removed from %s in %d minute%s.",
+			uh, args, temptime, (temptime == 1) ? "" : "s");
 		insert_ban(args, uh, temptime);
 	} else
 		remove_ban(args, uh);
@@ -988,8 +1083,8 @@ char *args;
 	temptime = atoi(next_arg(args, &args));
 	sprintf(uh, "*!*@%s", cluster(stuff->host));
 	if (temptime > 0) {
-		say("Ban on %s will be removed in %d minute%s.",
-			uh, temptime, (temptime == 1) ? "" : "s");
+		say("Ban on %s will be removed from %s in %d minute%s.",
+			uh, args, temptime, (temptime == 1) ? "" : "s");
 		insert_ban(args, uh, temptime);
 	} else
 		remove_ban(args, uh);
@@ -1075,15 +1170,19 @@ char *args;
    send_to_server("JOIN %s %s", to, channel_key(to));
 }
 
-void tochanops(command, args)
-char *command;
-char *args;
+void
+tochanops(command, args)
+	char	*command;
+	char	*args;
 {
    char buffer[1024];
    char *to, *themsg;
    ChannelList *Chan;
    NickList *Nick;
    int nickcount;
+   int level;
+   int saidit = 0;
+   int method = ops_method;
 
    themsg = NULL;
    if (!(to=my_next_arg(&args)))
@@ -1106,51 +1205,140 @@ char *args;
       say("No message specified");
       return;
    }
-   Chan = lookup_channel(to, from_server, 0);
-   if (!Chan)
+   if (ops_method == OPS_METHOD_AUTO)
    {
-      say("Error looking up nicks for %s", to);
-      return;
+	if (get_server_options(from_server) & SOptions_WallChops)
+	{
+		method = OPS_METHOD_WALLCHOPS;
+	}
+	else if (get_server_options(from_server) & SOptions_AtChannel)
+	{
+		method = OPS_METHOD_HYBRID;
+	}
+	else
+	{
+		method = OPS_METHOD_DEFAULT;
+	}
    }
-   strcpy(buffer, "");
-   nickcount = 0;
-   for (Nick = Chan->nicks; Nick; Nick = Nick->next)
-   {
-      if (Nick->chanop &&
-          my_stricmp(Nick->nick, get_server_nickname(from_server)))
+   if (method == OPS_METHOD_DEFAULT) {
+      Chan = lookup_channel(to, from_server, 0);
+      if (!Chan)
       {
-         strcat(buffer, ",");
-         strcat(buffer, Nick->nick);
-	 nickcount++;
-	 if (nickcount >= get_int_var(MAX_WALL_NICKS_VAR)) {
-	    if ((*themsg == '-') && (*(themsg+1) == ' '))
-	    {
-	       (void) get_token(&themsg, " ");
-	       send_to_server("PRIVMSG %s :%s", buffer+1, themsg);
-	       say("[%s] Sent to ops on %s: %s", TimeStamp(), to, themsg);
+         say("Error looking up nicks for %s", to);
+         return;
+      }
+      strcpy(buffer, "");
+      nickcount = 0;
+      for (Nick = Chan->nicks; Nick; Nick = Nick->next)
+      {
+         if (Nick->chanop &&
+             my_stricmp(Nick->nick, get_server_nickname(from_server)))
+         {
+            strcat(buffer, ",");
+            strcat(buffer, Nick->nick);
+   	    nickcount++;
+   	    if (nickcount >= get_int_var(MAX_WALL_NICKS_VAR)) {
+	       if ((*themsg == '-') && (*(themsg+1) == ' '))
+	       {
+	          (void) get_token(&themsg, " ");
+	          send_to_server("PRIVMSG %s :%s", buffer+1, themsg);
+	          if (!saidit) {
+		     level = set_lastlog_msg_level(LOG_PUBLIC);
+	    	     message_from(to, LOG_MSG);
+	             say("[%s] Sent to ops on %s: %s", TimeStamp(), to, themsg);
+	    	     message_from(NULL, LOG_CURRENT);
+		     set_lastlog_msg_level(level);
+		     saidit = 1;
+	          }
+	       }
+	       else
+	       {
+	          send_to_server("NOTICE %s :\002[Wallop:%s]\002 %s", buffer+1, to, themsg);
+	          if (!saidit) {
+		     level = set_lastlog_msg_level(LOG_PUBLIC);
+	    	     message_from(to, LOG_MSG);
+	             say("[%s] Sent to ops on %s: \002[Wallop:%s]\002 %s", TimeStamp(), to, to, themsg);
+	    	     message_from(NULL, LOG_MSG);
+		     set_lastlog_msg_level(level);
+		     saidit = 1;
+	          }
+	       }
+	       nickcount = 0;
+   	       strcpy(buffer, "");
 	    }
-	    else
-	    {
-	       send_to_server("NOTICE %s :\002[Wallop:%s]\002 %s", buffer+1, to, themsg);
-	       say("[%s] Sent to ops on %s: \002[Wallop:%s]\002 %s", TimeStamp(), to, to, themsg);
-	    }
-	    nickcount = 0;
-   	    strcpy(buffer, "");
-	 }
+         }
+      }
+      if (strlen(buffer) > 0) {
+         if ((*themsg == '-') && (*(themsg+1) == ' '))
+         {
+            (void) get_token(&themsg, " ");
+            send_to_server("PRIVMSG %s :%s", buffer+1, themsg);
+            level = set_lastlog_msg_level(LOG_PUBLIC);
+	    message_from(to, LOG_MSG);
+            say("[%s] Sent to ops on %s: %s", TimeStamp(), to, themsg);
+	    message_from(NULL, LOG_MSG);
+ 	    set_lastlog_msg_level(level);
+         }
+         else
+         {
+            send_to_server("NOTICE %s :\002[Wallop:%s]\002 %s", buffer+1, to,
+		themsg);
+            level = set_lastlog_msg_level(LOG_PUBLIC);
+	    message_from(to, LOG_MSG);
+            say("[%s] Sent to ops on %s: \002[Wallop:%s]\002 %s", TimeStamp(),
+		to, to, themsg);
+	    message_from(NULL, LOG_MSG);
+ 	    set_lastlog_msg_level(level);
+         }
       }
    }
-   if (strlen(buffer) > 0) {
-      if ((*themsg == '-') && (*(themsg+1) == ' '))
-      {
-         (void) get_token(&themsg, " ");
-         send_to_server("PRIVMSG %s :%s", buffer+1, themsg);
-         say("[%s] Sent to ops on %s: %s", TimeStamp(), to, themsg);
-      }
-      else
-      {
-         send_to_server("NOTICE %s :\002[Wallop:%s]\002 %s", buffer+1, to, themsg);
-         say("[%s] Sent to ops on %s: \002[Wallop:%s]\002 %s", TimeStamp(), to, to, themsg);
+   else if (method == OPS_METHOD_WALLCHOPS)
+   {
+      int list_type;
+      int format_type;
+
+      if (is_current_channel(to, 0)) {
+	 list_type = SEND_OPS_LIST;
+	 format_type = SEND_OPS_FMT;
+      } else {
+	 list_type = SEND_OPS_OTHER_LIST;
+	 format_type = SEND_OPS_OTHER_FMT;
       }
+      level = set_lastlog_msg_level(LOG_PUBLIC);
+      message_from(to, LOG_MSG);
+      if (do_hook(list_type, "%s %s", to, themsg))
+         put_it("%s", parseformat(format_type,
+            get_server_nickname(get_window_server(0)), to,
+            themsg));
+      message_from(NULL, LOG_MSG);
+      set_lastlog_msg_level(level);
+      send_to_server("WALLCHOPS %s :%s", to, themsg);
+   }
+   else if (method == OPS_METHOD_HYBRID)
+   {
+      int list_type;
+      int format_type;
+
+      if (is_current_channel(to, 0)) {
+	 list_type = SEND_OPS_LIST;
+	 format_type = SEND_OPS_FMT;
+      } else {
+	 list_type = SEND_OPS_OTHER_LIST;
+	 format_type = SEND_OPS_OTHER_FMT;
+      } 
+      level = set_lastlog_msg_level(LOG_PUBLIC);
+      message_from(to, LOG_MSG);
+      if (do_hook(list_type, "%s %s", to, themsg))
+         put_it("%s", parseformat(format_type,
+            get_server_nickname(get_window_server(0)), to,
+            themsg));
+      message_from(NULL, LOG_MSG);
+      set_lastlog_msg_level(level);
+      send_to_server("PRIVMSG @%s :%s", to, themsg);
+   }
+   else
+   {
+      say("Help!  I'm confused!  :-(");
    }
 }
 
@@ -1180,7 +1368,8 @@ char *args;
       say("No object specified to bomb");
 }
 
-void nslookup(command, args)
+void
+nslookup(command, args)
 char *command;
 char *args;
 {
@@ -1190,7 +1379,7 @@ char *args;
   {
         say("Checking tables...");
         if (!strchr(host, '.'))
-	   add_to_whois_queue(host, userhost_nsl, 0);
+	   typed_add_to_whois_queue(WHOIS_USERHOST, host, userhost_nsl, 0);
 	else
         {
            hostname=do_nslookup(host);
@@ -1223,7 +1412,6 @@ char *command,
 {
         char    *newhname;
         struct hostent *hp;
-	char	crap[1024];
 
         if (newhname = next_arg(args, &args))
         {
@@ -1232,9 +1420,7 @@ char *command,
                 {
                         bcopy(hp->h_addr, (char *) &MyHostAddr, sizeof(MyHostAddr));
                         local_ip_address.s_addr = ntohl(MyHostAddr.s_addr);
-			sprintf(crap, "MYHOSTADDR=%s", MyHostName);
-			/*setenv("MYHOSTADDR", MyHostName, 1);*/
-			putenv(crap);
+			set_string_var(VHOST_VAR, MyHostName);
                 }
 		do_reconnect(NULL);
         }
@@ -1731,16 +1917,16 @@ char *args;
 			strcat(tempbuf, "*@");
 		strcat(tempbuf, spec);
 		if (temptime > 0) {
-			say("Ban on %s will be removed in %d minute%s.",
-				tempbuf, temptime, (temptime == 1) ? "" : "s");
+			say("Ban on %s will be removed from %s in %d minute%s.",
+				tempbuf, to, temptime, (temptime == 1) ? "" : "s");
 			insert_ban(to, tempbuf, temptime);
 		} else
 			remove_ban(to, tempbuf);
 		send_to_server("MODE %s +b %s", to, tempbuf);
 	} else {
 		if (temptime > 0) {
-			say("Ban on %s will be removed in %d minute%s.",
-				spec, temptime, (temptime == 1) ? "" : "s");
+			say("Ban on %s will be removed from %s in %d minute%s.",
+				spec, to, temptime, (temptime == 1) ? "" : "s");
 			insert_ban(to, spec, temptime);
 		} else
 			remove_ban(to, spec);
@@ -2120,8 +2306,8 @@ char *args;
 			send_to_server("MODE %s -o+b %s %s", to,
 				nicks->nick, uh);
 			if (temptime > 0) {
-				say("Ban on %s will be removed in %d minute%s.",
-					uh, temptime, (temptime == 1) ? "" : "s");
+				say("Ban on %s will be removed from %s in %d minute%s.",
+					uh, to, temptime, (temptime == 1) ? "" : "s");
 				insert_ban(to, uh, temptime);
 			} else
 				remove_ban(to, uh);
@@ -2193,8 +2379,8 @@ char *args;
 			send_to_server("MODE %s -o+b %s %s", to,
 				nicks->nick, uh);
 			if (temptime > 0) {
-				say("Ban on %s will be removed in %d minute%s.",
-					uh, temptime, (temptime == 1) ? "" : "s");
+				say("Ban on %s will be removed from %s, in %d minute%s.",
+					uh, to, temptime, (temptime == 1) ? "" : "s");
 				insert_ban(to, uh, temptime);
 			} else
 				remove_ban(to, uh);
@@ -2260,8 +2446,8 @@ char *args;
 		else
 			strcpy((char *)&tmp, spec);
 		if (temptime > 0) {
-			say("Ban on %s will be removed in %d minute%s.",
-				tmp, temptime, (temptime == 1) ? "" : "s");
+			say("Ban on %s will be removed from %s in %d minute%s.",
+				tmp, to, temptime, (temptime == 1) ? "" : "s");
 			insert_ban(to, tmp, temptime);
 		} else
 			remove_ban(to, tmp);
@@ -2289,8 +2475,8 @@ char *args;
 				temp++;
 			sprintf(tmp, "*!*%s@%s", temp, cluster(nicks->host));
 			if (temptime > 0) {
-				say("Ban on %s will be removed in %d minute%s.",
-					tmp, temptime, (temptime == 1) ? "" : "s");
+				say("Ban on %s will be removed from %s in %d minute%s.",
+					tmp, to, temptime, (temptime == 1) ? "" : "s");
 				insert_ban(to, tmp, temptime);
 			} else
 				remove_ban(to, tmp);
@@ -2363,8 +2549,8 @@ char *args;
 		{
 			sprintf(tmp, "*!*@%s", cluster(nicks->host));
 			if (temptime > 0) {
-				say("Ban on %s will be removed in %d minute%s.",
-					tmp, temptime, (temptime == 1) ? "" : "s");
+				say("Ban on %s will be removed from %s in %d minute%s.",
+					tmp, to, temptime, (temptime == 1) ? "" : "s");
 				insert_ban(to, tmp, temptime);
 			} else
 				remove_ban(to, tmp);
@@ -2603,3 +2789,9 @@ char *args;
         send_to_server("TRACE %s", server);
 }
 
+void
+set_ops_method(value)
+	int	value;
+{
+	ops_method = value;
+}
