@@ -1,15 +1,15 @@
 /*
  * dcc.c: Things dealing client to client connections. 
  *
- * Written By Troy Rollo <troy@cbme.unsw.oz.au> 
+ * Written By Timothy Jensen
  *
- * Copyright(c) 1991 
+ * Copyright (c) 1999-2001 
  *
- * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
+ * See the COPYRIGHT file, or do a /HELP COPYRIGHT 
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: dcc.c,v 1.1.1.1 1999/07/16 21:21:42 toast Exp $";
+static	char	rcsid[] = "@(#)$Id: dcc.c,v 1.4 2001/04/01 15:58:26 toast Exp $";
 #endif
 
 #include "irc.h"
@@ -77,6 +77,8 @@ extern	void	process_incoming_talk();
 extern	void	process_incoming_raw();
 extern	void	process_incoming_listen();
 
+static 	void	dcc_auto_connect();
+
 struct
 {
 	char	*name;
@@ -1129,12 +1131,61 @@ register_dcc_offer(user, type, description, address, port, size)
 		dcc_chat(user);
 	}
 	else if (Client->filesize)
+	{
 		say("DCC %s (%s %d) request received from %s", type, description, Client->filesize, user);
+		if (get_int_var(AUTO_ACCEPT_DCC_SEND_VAR))
+			dcc_auto_connect(Client);
+	}
 	else
+	{
 		say("DCC %s (%s) request received from %s", type, description, user);
+		if (get_int_var(AUTO_ACCEPT_DCC_CHAT_VAR))
+			dcc_auto_connect(Client);
+	}
 	return;
 }
 
+static void
+dcc_auto_connect(Client)
+DCC_list	*Client;
+{
+	char *fullname;
+	int type;
+
+	if (!Client)
+		return;
+
+	if (!(Client->flags & DCC_OFFER))
+		return;
+
+	type = Client->flags & DCC_TYPES;
+
+	if (type == DCC_FILEREAD)
+		Client->bytes_sent = Client->bytes_read = 0L;
+	else if (type != DCC_CHAT)
+		return;
+
+	Client->flags |= DCC_TWOCLIENTS;
+
+	if (!dcc_open(Client))
+		return;
+
+	if (type == DCC_FILEREAD)
+	{
+		if (0 == (fullname = expand_twiddle(Client->description)))
+			malloc_strcpy(fullname, Client->description);
+		if (-1 == (Client->file = open(fullname,
+			O_WRONLY | O_TRUNC | O_CREAT, 0644)))
+		{
+			say("Unable to open %s: %s", Client->description,
+				errno ? strerror(errno) : "<No Error>");
+			close(Client->read);
+			dcc_erase(Client);
+		}
+		new_free(&fullname);
+	}
+}
+
 void
 process_incoming_chat(Client)
 	DCC_list	*Client;
@@ -1204,18 +1255,6 @@ process_incoming_chat(Client)
 		{
 			if (Client->user == NULL) /* chat closed in do_hook */
 				return;
-			if (away_set)
-			{
-				time_t	t;
-				char	*msg = (char *) 0;
-
-				t = time(0);
-				msg = (char *) new_malloc(strlen(tmp) + 20);
-				/*sprintf(msg, "%s <%.16s>", tmp, ctime(&t));*/
-				sprintf(msg, "%s", tmp);
-				strcpy(tmp, msg);
-				new_free(&msg);
-			}
 			if (*tmp == '\001')
 			{
 				char *ptr = tmp;
