@@ -2,15 +2,15 @@
  * edit.c: This is really a mishmash of function and such that deal with IRCII
  * commands (both normal and keybinding commands) 
  *
- * Written By Michael Sandrof
+ * Written By Timothy Jensen
  *
- * Copyright(c) 1990 
+ * Copyright(c) 1999-2001 
  *
- * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
+ * See the COPYRIGHT file, or do a /HELP COPYRIGHT 
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: edit.c,v 1.7 1999/07/26 03:15:39 toast Exp $";
+static	char	rcsid[] = "@(#)$Id: edit.c,v 1.19 2001/12/09 04:38:13 toast Exp $";
 #endif
 
 #include "irc.h"
@@ -59,6 +59,7 @@ static	char	rcsid[] = "@(#)$Id: edit.c,v 1.7 1999/07/26 03:15:39 toast Exp $";
 #include "recorder.h"
 #include "fileio.h"
 #include "autoop.h"
+#include "rejoin.h"
 
 /*
  * current_exec_timer - used to make sure we don't remove a timer
@@ -302,7 +303,6 @@ static	IrcCommand irc_command[] =
 	{ "ME",		NULL,		me,			0 },
 	{ "MK",		NULL,		masskick,		0 },
 	{ "MKB",	NULL,		masskickban,		0 },
-	{ "MKBOLD",	NULL,		masskickbanold,		0 },
 	{ "MLIST",	NULL,		mlist,			0 },
 	{ "MLOAD",	NULL,		mload,			0 },
 	{ "MODE",	"MODE",		send_channel_com,	0 },
@@ -342,10 +342,12 @@ static	IrcCommand irc_command[] =
 	{ "RCHAT",	NULL,		rchat_cmd,		0 },
 	{ "REDIRECT",	NULL,		redirect,		0 },
 	{ "REHASH",	"REHASH",	send_comm,		0 },
+	{ "REJOIN",	NULL,		rejoin,			0 },
 	{ "RELM",       "RELM",		rel_msg,		0 },
 	{ "RELN",       "RELN",		rel_msg,		0 },
 	{ "RELS",       "RELS",		rel_msg,		0 },
 	{ "RELSN",	"RELSN",	rel_msg,		0 },
+	{ "REPEAT",	"REPEAT",	repeatcmd,		0 },
 	{ "REQUEST",	NULL,		ctcp,			0 },
 	{ "RESET",	NULL,		reset,			0 },
 	{ "RESTART",	"RESTART",	send_comm,		0 },
@@ -366,7 +368,9 @@ static	IrcCommand irc_command[] =
 	{ "STACK",	NULL,		stackcmd,		0 },
 	{ "STATS",	"STATS",	send_comm,		0 },
 	{ "SUMMON",	"SUMMON",	send_comm,		0 },
+	{ "SWITCH",	"SWITCH",	switchcmd,		0 },
 	{ "T",		NULL,		dotopic,		0 },
+/*	{ "TEST", 	NULL,		toasttest,		0 },*/
 	{ "TIME",	"TIME",		send_comm,		0 },
 	{ "TIMER",	"TIMER",	timercmd,		0 },
 	{ "TKLINE",	"TKLINE",	dokline,		0 },
@@ -564,11 +568,9 @@ deop(command, args, subargs)
 		*subargs;
 {
 	send_to_server("MODE %s -o", get_server_nickname(from_server));
-#ifdef REJOIN_OPER
-	set_server_operator(from_server,0);
-	set_server_operator_pwd(from_server,(char *)0);
-	set_server_operator_nick(from_server,(char *)0);
-#endif /* EHUD */
+	set_server_operator(from_server, NULL);
+	clear_server_operator_pwd(from_server, NULL);
+	set_server_operator_nick(from_server, NULL);
 }
 
 static	void
@@ -864,10 +866,11 @@ oper_password_received(data, line)
 	char	*line;
 {
 	send_to_server("OPER %s %s", data, line);
-#ifdef REJOIN_OPER
-        set_server_operator_nick(from_server,data);
-        set_server_operator_pwd(from_server,line);
-#endif /* EHUD */
+	if (get_int_var(AUTO_REOPER_VAR))
+	{
+        	set_server_operator_nick(from_server, data);
+        	set_server_operator_pwd(from_server, line);
+	}
 }
 
 /* oper: the OPER command.  */
@@ -891,10 +894,11 @@ oper(command, args, subargs)
 		return;
 	}
 	send_to_server("OPER %s %s", nick, password);
-#ifdef REJOIN_OPER
-	set_server_operator_nick(from_server,nick);
-	set_server_operator_pwd(from_server,password);
-#endif /* EHUD */
+	if (get_int_var(AUTO_REOPER_VAR))
+	{
+		set_server_operator_nick(from_server, nick);
+		set_server_operator_pwd(from_server, password);
+	}
 }
 
 /* Full scale abort.  Does a "save" into the filename in line, and
@@ -1170,6 +1174,11 @@ info(command, args, subargs)
 {
 	if (!args || !*args)
 	{
+		say("Blackened IRC: written by Timothy L. Jensen");
+		say("       E-Mail: toast@blackened.com  IRC: Toast");
+		say("       Copyright (C) 1999-2001");
+		say("       Based on ircii-2.8.2.  See below for details.");
+		say("");
 		say("ircii: originally written by Michael Sandrof");
 		say("       versions 2.1 to 2.2pre7 by Troy Rollo");
 		say("       development continued by matthew green");
@@ -1209,7 +1218,7 @@ ison_ison(notused, nicklist)
 	char	*nicklist;
 {
 	if (do_hook(current_numeric, "%s", nicklist))
-		put_it("%s Currently online: %s", numeric_banner(), nicklist);
+		put_it("%sCurrently online: %s", numeric_banner(), nicklist);
 }
 
 static	void
@@ -1681,6 +1690,7 @@ e_quit(command, args, subargs)
 		{
 			from_server = i;
 			send_to_server("QUIT :%s", Reason);
+			put_it("%s", parseformat(QUIT_FMT, get_server_nickname(get_window_server(0)), Reason));
 		}
 	irc_exit();
 }
@@ -1711,42 +1721,33 @@ e_wall(command, args, subargs)
 		*args,
 		*subargs;
 {
-/* This is crap
-	if (strcmp(command, "WALL") == 0)
-	{
-		message_from(NULL, LOG_WALL);
-		if (!get_server_operator(from_server))
-			put_it("## %s", args);
-	}
-	else
-	{
-		message_from(NULL, LOG_WALLOP);
-		if (!get_server_flag(from_server, USER_MODE_W))
-			put_it("!! %s", args);
-	}*/
+	int level;
+
 	if (!in_on_who)
 		send_to_server("%s :%s", command, args);
 	if (get_int_var(SHOW_MY_WALLOPS_VAR)) {
-		Window  *old_to_window;
 		message_from(NULL, LOG_WALLOP);
-		old_to_window = to_window;
-		to_window = get_window_by_name(get_string_var(SERVER_NOTICE_WINDOW_VAR));
-
+		level = set_lastlog_msg_level(LOG_WALLOP);
 		if (!strcasecmp(command, "OPERWALL"))
 			put_it("%s", parseformat(SEND_OPERWALL_FMT,
-				get_server_nickname(get_window_server(0)),
-				args));
+				get_server_nickname(get_window_server(0)), args,
+				get_server_operator(get_window_server(0)) ? "*"
+				: empty_string));
 		else if (!strcasecmp(command, "LOCOPS"))
 			put_it("%s", parseformat(SEND_LOCOPS_FMT,
-				get_server_nickname(get_window_server(0)),
-				args));
+				get_server_nickname(get_window_server(0)), args,
+				get_server_operator(get_window_server(0)) ? "*"
+				: empty_string));
 		else
 			put_it("%s", parseformat(SEND_WALLOPS_FMT,
-				get_server_nickname(get_window_server(0)),
-				args));
-		to_window = old_to_window;
+				get_server_nickname(get_window_server(0)), args,
+				get_server_operator(get_window_server(0)) ? "*"
+				: empty_string));
+		if (beep_on_level & LOG_WALLOP)
+			beep_em(1);
+		set_lastlog_msg_level(level);
+		message_from(NULL, LOG_CRAP);
 	}
-	message_from(NULL, LOG_CRAP);
 }
 
 /*
@@ -3139,7 +3140,7 @@ type_text(c, str)
 		*str;
 {
 	for (; *str; str++)
-		input_add_character(*str);
+		input_add_character(*str, NULL);
 }
 
 /*
@@ -3251,7 +3252,7 @@ edit_char(key)
 			{
 				if ((extended_key =
 				    get_digraph(extended_key)) != '\0')
-					input_add_character(extended_key);
+					input_add_character(extended_key, NULL);
 				else
 					term_beep();
 			}
@@ -3259,7 +3260,7 @@ edit_char(key)
 		else if (current_screen->quote_hit)
 		{
 			current_screen->quote_hit = 0;
-			input_add_character(extended_key);
+			input_add_character(extended_key, NULL);
 		}
 		else if (func)
 			func(extended_key, ptr ? ptr : empty_string);
@@ -3822,13 +3823,13 @@ xtypecmd(command, args, subargs)
 			if (!my_strnicmp(arg, "LITERAL", len))
 			{
 				for (; *args; args++)
-					input_add_character(*args);
+					input_add_character(*args, NULL);
 			}
 			else
 				say ("Unknown flag -%s to XTYPE", arg);
 			return;
 		}
-		input_add_character('-');
+		input_add_character('-', NULL);
 	}
 	else
 		type(command, args, empty_string);
