@@ -9,7 +9,7 @@
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: exec.c,v 1.1.1.1 1999/07/16 21:21:42 toast Exp $";
+static	char	rcsid[] = "@(#)$Id: exec.c,v 1.2 2001/11/29 19:31:11 toast Exp $";
 #endif
 
 #define IN_EXEC_C
@@ -783,7 +783,8 @@ start_process(name, logical, redirect, who, refnum)
 		cnt;
 	char	*shell,
 		*flag,
-		*arg;
+		*arg,
+		*banner;
 
 #ifdef DAEMON_UID
 	if (getuid() == DAEMON_UID)
@@ -882,7 +883,9 @@ start_process(name, logical, redirect, who, refnum)
 			setgid(getgid());
 			execl(shell, shell, flag, name, (char *) 0);
 		}
-		sprintf(buffer, "*** Error starting shell \"%s\": %s\n", shell,
+		banner = get_string_var(BANNER_VAR);
+		sprintf(buffer, "%sError starting shell \"%s\": %s\n", 
+			banner ? banner : empty_string, shell,
 			strerror(errno));
 		write(1, buffer, strlen(buffer));
 		_exit(-1);
