@@ -4,10 +4,11 @@
  * Written by Michael Sandrof
  * Extensively modified by Troy Rollo
  * Re-modified by Matthew Green
+ * Re-re-modified by Timothy Jensen
  *
- * Copyright(c) 1992 
+ * Copyright(c) 1992-2001 
  *
- * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
+ * See the COPYRIGHT file, or do a /HELP COPYRIGHT 
  */
 
 /*
@@ -24,11 +25,15 @@
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: help.c,v 1.6 1999/08/22 19:05:17 toast Exp $";
+static	char	rcsid[] = "@(#)$Id: help.c,v 1.12 2001/11/29 19:31:11 toast Exp $";
 #endif
 
 #include "irc.h"
 
+#include "help.h"
+#include "vars.h"
+#include "ircaux.h"
+
 /* stuff from gnu autoconf docs */
 
 #ifdef NeXT		/* ugly hack 'cause configure don't grok it -phone */
@@ -56,13 +61,10 @@ static	char	rcsid[] = "@(#)$Id: help.c,v 1.6 1999/08/22 19:05:17 toast Exp $";
 
 #include "term.h"
 #include "server.h"
-#include "vars.h"
-#include "ircaux.h"
 #include "input.h"
 #include "window.h"
 #include "screen.h"
 #include "output.h"
-#include "help.h"
 
 #if defined(ISC22)
 extern char *strrchr();
@@ -85,11 +87,13 @@ static	void	help_show_paused_topic _((char *));
 static	void	create_help_window _((void));
 static	void	set_help_screen _((Screen *));
 static	int	use_helpfile _((void));
-static	char	*help_fgets _((char *, int, FILE *));
+static	void	help_fclose _((void));
+static	char	*help_fgets _((char *, int));
 static	int	help_seek _((char *));
 static	int	help_scan _((char *));
 static	int	is_super_topic _((char *));
 static	int	help_index _((char *, IEntry **));
+static	void	clear_index _((IEntry **));
 
 /*
  * A few variables here - A lot added to get help working with
@@ -97,7 +101,7 @@ static	int	help_index _((char *, IEntry **));
  * reading things from the server(s), so not to ping timeout.
  */
 static	Window	*help_window = (Window *) 0;
-static	FILE	*help_fp;
+static	FILE	*help_fp = NULL;
 static	char	no_help[] = "NOHELP";
 static	int	entry_size;
 static	char	*this_arg;
@@ -179,7 +183,7 @@ show_help(window, name)
 		rows = window->display_size;
 	while (--rows)
 	{
-		if (help_fgets(line, 80, help_fp))
+		if (help_fgets(line, 80))
 		{
 			if (*(line + strlen(line) - 1) == '\n')
 			*(line + strlen(line) - 1) = (char) 0;
@@ -214,7 +218,7 @@ show_help(window, name)
 		}
 		else
 		{
-			fclose(help_fp);
+			help_fclose();
 			return (0);
 		}
 	}
@@ -248,7 +252,7 @@ help_prompt(name, line)
 	if (line && ((*line == 'q') || (*line == 'Q')))
 	{
 		finished_help_paging = 1;
-		fclose(help_fp);
+		help_fclose();
 		set_help_screen((Screen *) 0);
 		return;
 	}
@@ -262,7 +266,7 @@ help_prompt(name, line)
 	else
 	{
 		finished_help_paging = 1;
-		fclose(help_fp);
+		help_fclose();
 		if (help_show_directory)
 		{
 			if (get_int_var(HELP_PAGER_VAR))
@@ -316,6 +320,7 @@ help_topic(path, name)
 {
 	struct	stat	stat_buf;
 	char	filename[BIG_BUFFER_SIZE];
+	char	*banner = get_string_var(BANNER_VAR);
 
 #ifdef ZCAT
 	char	*name_z = (char *) 0;
@@ -348,7 +353,7 @@ help_topic(path, name)
 			if (stat_file(filename, &stat_buf) == -1)
 			{
 				help_put_it(name,
-					"*** No help available on %s: Use ? for list of topics", name);
+					"%sNo help available on %s: Use ? for list of topics", banner ? banner : empty_string, name);
 				return;
 			}
 			else
@@ -374,7 +379,7 @@ help_topic(path, name)
 			if ((help_fp = fopen(filename, "r")) == (FILE *) 0)
 			{
 				help_put_it(name,
-					"*** No help available on %s: Use ? for list of topics", name);
+					"%sNo help available on %s: Use ? for list of topics", banner ? banner : empty_string, name);
 				return;
 			}
 #ifdef ZCAT
@@ -385,7 +390,7 @@ help_topic(path, name)
 			if ((help_fp = zcat(filename)) == (FILE *) 0)
 			{
 				help_put_it(name,
-					"*** No help available on %s: Use ? for list of topics", name);
+					"%sNo help available on %s: Use ? for list of topics", banner ? banner : empty_string, name);
 				return;
 			}
 		}
@@ -405,14 +410,14 @@ help_topic(path, name)
 		if ((help_fp = fopen(get_string_var(HELP_FILE_VAR), "r")) == (FILE *) 0)
 		{
 			help_put_it(name,
-				"*** No help available on %s: Use ? for list of topics", name);
+				"%sNo help available on %s: Use ? for list of topics", banner ? banner : empty_string, name);
 			return;
 		}
 		if (help_seek(name))
 		{
 			help_put_it(name,
-				"*** No help available on %s: Use ? for list of topics", name);
-			fclose(help_fp);
+				"%sNo help available on %s: Use ? for list of topics", banner ? banner : empty_string, name);
+			help_fclose();
 			return;
 		}
 	}
@@ -423,7 +428,7 @@ help_topic(path, name)
 	 * the first time.
 	 */
 
-	help_put_it(name, "*** Help on %s", name);
+	help_put_it(name, "%sHelp on %s", banner ? banner : empty_string, name);
 	help_prompt(name, (char *) 0);
 }
 
@@ -532,6 +537,7 @@ help_me(topics, args)
 	int	use_file;
 	char	*tmp_arg;
 	IEntry	*items, *tmpitem;
+	char	*banner = get_string_var(BANNER_VAR);
 
 #ifdef ZCAT
 	char	*arg_z = (char *) 0;
@@ -580,7 +586,8 @@ help_me(topics, args)
 		if (access(path, R_OK|X_OK))
 		{
 			help_put_it(no_help,
-				"*** Cannot access help directory!");
+				"%sCannot access help directory!",
+				banner ? banner : empty_string);
 			set_help_screen((Screen *) 0);
 			return;
 		}
@@ -664,7 +671,8 @@ help_me(topics, args)
 			switch (entries)
 			{
 			case -1:
-				help_put_it(no_help, "*** Error during help function: %s", strerror(errno));
+				banner = get_string_var(BANNER_VAR);
+				help_put_it(no_help, "%sError during help function: %s", banner ? banner : empty_string, strerror(errno));
 				set_help_screen((Screen *) 0);
 				if (help_paused_first_call)
 				{
@@ -675,7 +683,8 @@ help_me(topics, args)
 				}
 				return;
 			case 0:
-				help_put_it(this_arg, "*** No help available on %s: Use ? for list of topics", this_arg);
+				banner = get_string_var(BANNER_VAR);
+				help_put_it(this_arg, "%sNo help available on %s: Use ? for list of topics", banner ? banner : empty_string, this_arg);
 				if (!get_int_var(HELP_PROMPT_VAR))
 				{
 					set_help_screen((Screen *) 0);
@@ -736,7 +745,7 @@ help_me(topics, args)
 			default:
 				help_show_directory = 1;
 				strcpy(paused_topic, help_topic_list);
-				help_pause_add_line("*** %s choices:", help_topic_list);
+				help_pause_add_line("%s%s choices:", banner ? banner : empty_string, help_topic_list);
 				*buffer = (char) 0;
 				cnt = 0;
 				entry_size += 2;
@@ -858,7 +867,7 @@ help_me(topics, args)
 				dont_pause_topic = 1;
 			switch(entries) {
 			case -1:
-				help_put_it(no_help, "*** Error during help function: %s", strerror(errno));
+				help_put_it(no_help, "%sError during help function: %s", banner ? banner : empty_string, strerror(errno));
 				set_help_screen((Screen *) 0);
 				if (help_paused_first_call)
 				{
@@ -869,7 +878,7 @@ help_me(topics, args)
 				}
 				return;
 			case 0:
-				help_put_it(this_arg, "*** No help available on %s: Use ? for list of topics", this_arg);
+				help_put_it(this_arg, "%sNo help available on %s: Use ? for list of topics", banner ? banner : empty_string, this_arg);
 				if (!get_int_var(HELP_PROMPT_VAR))
 				{
 					set_help_screen((Screen *) 0);
@@ -887,10 +896,12 @@ help_me(topics, args)
 					new_free(&help_paused_path);
 					new_free(&help_paused_name);
 				}
+				/*
 				for (i = 0; i < free_cnt; i++)
 				{
 					new_free(&namelist[i]);
 				}
+				*/
 				break;
 			case 1:
 				if (is_super_topic(this_arg))
@@ -924,7 +935,7 @@ help_me(topics, args)
 			default:
 				help_show_directory = 1;
 				strcpy(paused_topic, help_topic_list);
-				help_pause_add_line("*** %s choices:", help_topic_list);
+				help_pause_add_line("%s%s choices:", banner? banner : empty_string, help_topic_list);
 				items = NULL;
 				entries = help_index(help_topic_list, &items);
 
@@ -990,6 +1001,7 @@ help(command, args, subargs)
 {
 	char	*help_file;
 	char	*help_path;
+	char	*banner = get_string_var(BANNER_VAR);
 
 	finished_help_paging = 0;
 	help_show_directory = 0;
@@ -1018,7 +1030,7 @@ help(command, args, subargs)
 		if (!(help_path && *help_path &&
 			!access(help_path, R_OK | X_OK)))
 		{
-			help_put_it(no_help, "*** No HELP_FILE or HELP_PATH variable set");
+			help_put_it(no_help, "%sNo HELP_FILE or HELP_PATH variable set", banner ? banner : empty_string);
 			return;
 		}
 	}
@@ -1101,26 +1113,38 @@ use_helpfile()
 	return 0;
 }
 
+static void
+help_fclose(void)
+{
+	if (help_fp)
+	{
+		fclose(help_fp);
+		help_fp = NULL;
+	}
+}
+
 static	char	*
-help_fgets(str, size, stream)
+help_fgets(str, size)
 	char	*str;
 	int	size;
-	FILE	*stream;
 {
 	char	*tmp;
 	char	buf[82];
 
-	tmp = get_string_var(HELP_FILE_VAR);
+	if (!help_fp)
+	{
+		return (char *)NULL;
+	}
 
 	if (use_helpfile()) {
-		tmp = fgets(buf, 81, stream);
+		tmp = fgets(buf, 81, help_fp);
 		if (tmp && (*buf == '\t')) {
 			bzero(str, size);
 			strncpy(str, buf+1, size-1);
 		} else
 			return (char *)NULL;
 	} else
-		return fgets(str, size, stream);
+		return fgets(str, size, help_fp);
 }
 
 static	int
@@ -1154,7 +1178,7 @@ help_scan(topic)
 	if (topic && *topic && (topic[strlen(topic)-1] != '?') &&
 			!help_seek(topic))
 	{
-		fclose(help_fp);
+		help_fclose();
 		return 1;
 	}
 
@@ -1167,11 +1191,11 @@ help_scan(topic)
 	} else
 		sprintf(tmp, "%s", topic);
 	if (!help_seek(tmp)) {
-		fclose(help_fp);
+		help_fclose();
 		return 2;
 	}
 
-	fclose(help_fp);
+	help_fclose();
 	return 0;
 }
 
@@ -1190,11 +1214,11 @@ is_super_topic(topic)
 	else
 		tmp[0] = '?';
 	if (!help_seek(tmp)) {
-		fclose(help_fp);
+		help_fclose();
 		return 1;
 	}
 
-	fclose(help_fp);
+	help_fclose();
 	return 0;
 }
 
@@ -1216,13 +1240,13 @@ help_index(topic, entries)
 	else
 		tmp[0] = '?';
 	if (help_seek(tmp)) {
-		fclose(help_fp);
+		help_fclose();
 		return 0;
 	}
 
 	entry_size = cnt = 0;
 
-	while (help_fgets(buf, 80, help_fp)) {
+	while (help_fgets(buf, 80)) {
 		cnt++;
 		l = strlen(buf);
 		if ((l>0) && (buf[l-1] == '\n'))
@@ -1248,7 +1272,7 @@ help_index(topic, entries)
 		}
 	}
 
-	fclose(help_fp);
+	help_fclose();
 	return cnt;
 }
 
