@@ -9,7 +9,7 @@
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: hook.c,v 1.2 1999/07/25 22:31:26 toast Exp $";
+static	char	rcsid[] = "@(#)$Id: hook.c,v 1.5 2001/11/29 19:31:11 toast Exp $";
 #endif
 
 #include "irc.h"
@@ -93,6 +93,8 @@ extern	int	load_depth;
 	{ "NOTICE",		(Hook *) 0,	2,	0,	0 },
 	{ "NOTIFY_SIGNOFF",	(Hook *) 0,	1,	0,	0 },
 	{ "NOTIFY_SIGNON",	(Hook *) 0,	1,	0,	0 },
+	{ "OPS",		(Hook *) 0,	3,	0,	0 },
+	{ "OPS_OTHER",		(Hook *) 0,	3,	0,	0 },
 	{ "PUBLIC",		(Hook *) 0,	3,	0,	0 },
 	{ "PUBLIC_MSG",		(Hook *) 0,	3,	0,	0 },
 	{ "PUBLIC_NOTICE",	(Hook *) 0,	3,	0,	0 },
@@ -102,6 +104,8 @@ extern	int	load_depth;
 	{ "SEND_DCC_CHAT",	(Hook *) 0,	2,	0,	0 },
 	{ "SEND_MSG",		(Hook *) 0,	2,	0,	0 },
 	{ "SEND_NOTICE",	(Hook *) 0,	2,	0,	0 },
+	{ "SEND_OPS",		(Hook *) 0,	2,	0,	0 },
+	{ "SEND_OPS_OTHER",	(Hook *) 0,	2,	0,	0 },
 	{ "SEND_PUBLIC",	(Hook *) 0,	2,	0,	0 },
 	{ "SEND_TALK",		(Hook *) 0,	2,	0,	0 },
 	{ "SERVER_NOTICE",	(Hook *) 0,	1,	0,	0 },
@@ -710,14 +714,21 @@ on(command, args, subargs)
 		i,
 		flexible;
 	char	type;
+	char	*banner;
 
 	if (get_int_var(NOVICE_VAR) && !load_depth)
 	{
-	    yell("*** You may not type ON commands when you have the NOVICE");
-	    yell("*** variable set to ON. Some ON commands may cause a");
-	    yell("*** security breach on your machine, or enable another");
-	    yell("*** user to control your IRC session. Read the help files");
-	    yell("*** in /HELP ON before using ON");
+	    banner = get_string_var(BANNER_VAR);
+	    if (!banner)
+		banner = empty_string;
+	    yell("%sYou may not type ON commands when you have the NOVICE",
+		banner);
+	    yell("%svariable set to ON. Some ON commands may cause a", banner);
+	    yell("%ssecurity breach on your machine, or enable another",
+		banner);
+	    yell("%suser to control your IRC session. Read the help files",
+		banner);
+	    yell("%sin /HELP ON before using ON", banner);
 	    return;
 	}
 	if ((func = next_arg(args, &args)) != NULL)
