@@ -9,7 +9,7 @@
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: ircaux.c,v 1.2 1999/07/23 19:16:35 toast Exp $";
+static	char	rcsid[] = "@(#)$Id: ircaux.c,v 1.8 2001/12/03 02:55:57 toast Exp $";
 #endif
 
 #include "irc.h"
@@ -29,6 +29,7 @@ static	char	rcsid[] = "@(#)$Id: ircaux.c,v 1.2 1999/07/23 19:16:35 toast Exp $";
 #include "ircaux.h"
 #include "output.h"
 #include "term.h"
+#include "vars.h"
 
 extern	struct	in_addr	MyHostAddr;
 
@@ -402,6 +403,20 @@ malloc_strcpy(ptr, src)
 		*ptr = (char *) 0;
 }
 
+/* Like malloc_strcpy, but doesn't have to free anything first! :P */
+char	*
+m_strdup(src)
+	char	*src;
+{
+	char	*ptr;
+
+	if (!src)
+		src = empty_string;
+	
+	ptr = new_malloc(strlen(src) + 1);
+	return strcpy(ptr, src);
+}
+
 /* malloc_strcat: Yeah, right */
 void
 malloc_strcat(ptr, src)
@@ -584,7 +599,7 @@ connect_by_number(service, host)
 
         localaddr.sin_port   = htons(0);
 
-	if ((char *) 0 != (tmp = getenv("MYHOSTADDR"))) {
+	if ((char *) 0 != (tmp = get_string_var(VHOST_VAR))) {
            if (atoi(tmp) > 0) {
               /* Numeric -- use inet_addr */
               localaddr.sin_family = AF_INET;
@@ -965,12 +980,35 @@ strmcat_ue(dest, src, maxlen)
 	*dest = '\0';
 }
 
+/*
+ * my_strstr: Returns the first occurrance of str in source or NULL if there
+ * are no occurrances.
+ */
+char	*
+my_strstr(source, str)
+	char	*str,
+		*source;
+{
+	int	len;
+
+	len = strlen(str);
+	while (source && *source && str && *str)
+	{
+		if (!my_strnicmp(source, str, len))
+		{
+			return source;
+		}
+		source++;
+	}
+	return NULL;
+}
+
 /*
  * scanstr: looks for an occurrence of str in source.  If not found, returns
  * 0.  If it is found, returns the position in source (1 being the first
  * position).  Not the best way to handle this, but what the hell 
  */
-extern	int
+int
 scanstr(source, str)
 	char	*str,
 		*source;
@@ -989,6 +1027,30 @@ scanstr(source, str)
 	return (0);
 }
 
+/*
+ * countstr: Returns the number of occurrance of str in source.
+ */
+int
+countstr(source, str)
+	char	*str,
+		*source;
+{
+	int	i = 0,
+		len;
+
+	len = strlen(str);
+	while (source && *source && str && *str)
+	{
+		source = my_strstr(source, str);
+		if (source)
+		{
+			i++;
+			source += len;
+		}
+	}
+	return i;
+}
+
 /* expand_twiddle: expands ~ in pathnames. */
 char	*
 expand_twiddle(str)
@@ -1232,6 +1294,97 @@ double_quote(str, stuff)
 	return ptr;
 }
 
+char	*
+tdiff(ltime)
+	time_t	ltime;
+{
+	static char	result[80];
+	time_t		days,
+			hours,
+			minutes,
+			seconds;
+	char		*tstr;
+
+	seconds = ltime % 60;
+	ltime = (ltime - seconds) / 60;
+	minutes = ltime % 60;
+	ltime = (ltime - minutes) / 60;
+	hours = ltime % 24;
+	days = (ltime - hours) / 24;
+	tstr = result;
+	if (days)
+	{
+		sprintf(tstr, "%d day%s", (int)days,
+			(days == 1) ? empty_string : "s");
+		tstr += strlen(tstr);
+	}
+	if (hours)
+	{
+		sprintf(tstr, "%s%d hour%s", days ? " " : empty_string,
+			(int)hours, (hours == 1) ? empty_string : "s");
+		tstr += strlen(tstr);
+	}
+	if (minutes)
+	{
+		sprintf(tstr, "%s%d minute%s",
+			(hours || days) ? " " : empty_string, (int)minutes,
+			(minutes == 1) ? empty_string : "s");
+		tstr += strlen(tstr);
+	}
+	if (seconds || (!days && !hours && !minutes))
+	{
+		sprintf(tstr, "%s%d second%s",
+			(minutes || hours || days) ? " " : empty_string,
+			(int)seconds, (seconds == 1) ? empty_string : "s");
+		tstr += strlen(tstr);
+	}
+	return (char *)&result;
+}
+
+char  *
+tdiff2(ltime)
+	time_t	ltime;
+{
+	static char	result[80];
+	time_t		days,
+			hours,
+			minutes,
+			seconds;
+	char    	*tstr;
+
+	seconds = ltime % 60;
+	ltime = (ltime - seconds) / 60;
+	minutes = ltime % 60;
+	ltime = (ltime - minutes) / 60;
+	hours = ltime % 24;
+	days = (ltime - hours) / 24;
+	tstr = result;
+	if (days)
+	{
+		sprintf(tstr, "%dd", (int)days);
+		tstr += strlen(tstr);
+	}
+	if (hours)
+	{
+		sprintf(tstr, "%s%dh", days ? " " : empty_string, (int)hours);
+		tstr += strlen(tstr);
+	}
+	if (minutes)
+	{
+		sprintf(tstr, "%s%dm", (days || hours) ? " " : empty_string,
+			(int)minutes);
+		tstr += strlen(tstr);
+	}
+	if (seconds || (!days && !hours && !minutes))
+	{
+		sprintf(tstr, "%s%ds",
+			(days || hours || minutes) ? " " : empty_string,
+			(int)seconds);
+		tstr += strlen(tstr);
+	}
+	return (char *)&result;
+}
+
 /*
  * new_stty: given a string of stty commands sets the tty 
  *           via ioctls TCGETA/TCSETA.
