@@ -10,7 +10,7 @@
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: keys.c,v 1.1.1.1 1999/07/16 21:21:43 toast Exp $";
+static	char	rcsid[] = "@(#)$Id: keys.c,v 1.4 2001/12/09 05:28:30 toast Exp $";
 #endif
 
 #include "irc.h"
@@ -389,11 +389,11 @@ bindcmd(command, args, subargs)
 	char	*function;
 	int	meta;
 
-	if ((key = next_arg(args, &args)) != NULL)
+	if ((key = (u_char *)next_arg(args, &args)) != NULL)
 	{
 		if ((meta = parse_key(key)) == -1)
 			return;
-		if (strlen(key) > 1)
+		if (strlen((char *)key) > 1)
 		{
 			say("Key sequences may not contain more than two keys");
 			return;
@@ -635,7 +635,7 @@ KeyMap	keys[] =
 	{ QUIT_IRC,		0, 0,	(char *) 0 },
 	{ DELETE_CHARACTER,	0, 0,	(char *) 0 },
 	{ END_OF_LINE,		0, 0,	(char *) 0 },
-	{ FORWARD_CHARACTER,	0, 0,	(char *) 0 },
+	{ SELF_INSERT,		0, 0,	(char *) 0 },
 	{ SELF_INSERT,		0, 0,	(char *) 0 },
 
 	{ BACKSPACE,		0, 0,	(char *) 0 },	/* 8 */
