@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: names.c,v 1.2 1999/07/23 20:48:59 toast Exp $";
+static	char	rcsid[] = "@(#)$Id: names.c,v 1.4 2001/12/09 05:01:45 toast Exp $";
 #endif
 
 #include "irc.h"
@@ -98,9 +98,10 @@ lookup_channel(channel, server, unlink)
  * current channel as well 
  */
 void
-add_channel(channel, server)
+add_channel(channel, server, rejoin)
 	char	*channel;
-	int	server;
+	int	server,
+		rejoin;
 {
 	ChannelList *new;
 
@@ -133,7 +134,7 @@ add_channel(channel, server)
 		new->nicks = NULL;
 		add_to_list(&channel_list, new);
 	}
-	if (!is_current_channel(channel, 0))
+	if (!rejoin && !is_current_channel(channel, 0))
 	{
 		int	flag = 1;
 		Window	*tmp;
@@ -842,7 +843,7 @@ reconnect_all_channels()
 			if ((t = tmp->window->current_channel) && *t
 			&& !is_bound(t, tmp->window->server))
 				malloc_strcpy(&tmp->window->waiting_channel, t);
-			if (get_server_version(from_server) >= Server2_8)
+			if ((version = get_server_version(from_server)) >= Server2_8)
 			{
 				send_to_server("JOIN %s%s%s", tmp->channel, tmp->key ? " " : "", tmp->key ? tmp->key : "");
 				if ((char *) 0 != s)
