@@ -3,15 +3,15 @@
  * wacky, nutty program we call ircd 
  *
  *
- * written by michael sandrof
+ * Written by Timothy Jensen
  *
- * copyright(c) 1990 
+ * Copyright (c) 1999-2001 
  *
- * see the copyright file, or do a help ircii copyright 
+ * see the copyright file, or do a /help copyright 
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: numbers.c,v 1.7 1999/08/19 22:39:22 toast Exp $";
+static	char	rcsid[] = "@(#)$Id: numbers.c,v 1.14 2001/12/08 04:55:55 toast Exp $";
 #endif
 
 #include "irc.h"
@@ -46,18 +46,27 @@ static	char	tempstr[1024];
 
 /*
  * numeric_banner: This returns in a static string of either "xxx" where
- * xxx is the current numeric, or "***" if SHOW_NUMBERS is OFF 
+ * xxx is the current numeric, or the value of BANNER if SHOW_NUMBERS is OFF 
  */
 char	*
 numeric_banner()
 {
-	static	char	thing[4];
+	static	char	thing[5];
+	char	*foo = get_string_var(BANNER_VAR);
 
 	if (get_int_var(SHOW_NUMERICS_VAR))
-		sprintf(thing, "%3.3u", -current_numeric);
+	{
+		sprintf(thing, "%3.3u ", -current_numeric);
+		return (thing);
+	}
+	else if (foo)
+	{
+		return foo;
+	}
 	else
-		strcpy(thing, "***");
-	return (thing);
+	{
+		return empty_string;
+	}
 }
 
 /*
@@ -87,27 +96,27 @@ display_msg(from, ArgList)
 		if (strlen(rest))
 		{
 			if (from)
-				put_it("%s %s: %s (from %s)", numeric_banner(),
+				put_it("%s%s: %s (from %s)", numeric_banner(),
 					rest, ptr, from);
 			else
-				put_it("%s %s: %s", numeric_banner(), rest,
+				put_it("%s%s: %s", numeric_banner(), rest,
 					ptr);
 		}
 		else
 		{
 			if (from)
-				put_it("%s %s (from %s)", numeric_banner(),
+				put_it("%s%s (from %s)", numeric_banner(),
 					ptr, from);
 			else
-				put_it("%s %s", numeric_banner(), ptr);
+				put_it("%s%s", numeric_banner(), ptr);
 		}
 	}
 	else
 	{
 		if (from)
-			put_it("%s %s (from %s)", numeric_banner(), rest, from);
+			put_it("%s%s (from %s)", numeric_banner(), rest, from);
 		else
-			put_it("%s %s", numeric_banner(), rest);
+			put_it("%s%s", numeric_banner(), rest);
 	}
 }
 
@@ -236,7 +245,7 @@ channel_topic(from, ArgList)
 			return;
 		topic = ArgList[1];
 		message_from(channel, LOG_CRAP);
-		put_it("%s Topic for %s: %s", numeric_banner(), channel,
+		put_it("%sTopic for %s: %s", numeric_banner(), channel,
 			topic);
 	}
 	else
@@ -246,7 +255,7 @@ channel_topic(from, ArgList)
 		if (num_format_exists(-current_numeric))
 			display_msg(from, ArgList);
 		else
-			put_it("%s Topic: %s", numeric_banner(), ArgList[0]);
+			put_it("%sTopic: %s", numeric_banner(), ArgList[0]);
 	}
 }
 
@@ -293,13 +302,14 @@ not_valid_channel(from, ArgList)
 		if (num_format_exists(-current_numeric))
 			display_msg(from, ArgList);
 		else
-			put_it("%s %s %s", numeric_banner(), channel,
+			put_it("%s%s %s", numeric_banner(), channel,
 				ArgList[1]);
 	}
 }
 
 /* from ircd .../include/numeric.h */
 /*
+#define ERR_UNAVAILRESOURCE  437
 #define ERR_CHANNELISFULL    471
 #define ERR_INVITEONLYCHAN   473
 #define ERR_BANNEDFROMCHAN   474
@@ -311,8 +321,43 @@ cannot_join_channel(from, ArgList)
 	char	*from,	
 		**ArgList;
 {
-	if (ArgList[0])
-		remove_channel(ArgList[0], from_server);
+	char	*channel = ArgList[0];
+	int	rj = 0;
+
+	if (channel && is_channel(channel))
+	{
+		remove_channel(channel, from_server);
+		switch(-current_numeric)
+		{
+		case 437:
+			rj = rejoin_failed(channel, from_server,
+				REJOIN_UNAVAILABLE_VAR);
+			break;
+		case 471:
+			rj = rejoin_failed(channel, from_server,
+				REJOIN_FULL_VAR);
+			break;
+		case 473:
+			rj = rejoin_failed(channel, from_server,
+				REJOIN_INVITEONLY_VAR);
+			break;
+		case 474:
+			rj = rejoin_failed(channel, from_server,
+				REJOIN_BANNED_VAR);
+			break;
+		case 475:
+			rj = rejoin_failed(channel, from_server,
+				REJOIN_BADKEY_VAR);
+			break;
+		case 476:
+			/* Huh? */
+			break;
+		}
+		if (rj)
+		{
+			return;
+		}
+	}
 	PasteArgs(ArgList, 0);
 	if (num_format_exists(-current_numeric)) {
 		display_msg(from, ArgList);
@@ -321,6 +366,9 @@ cannot_join_channel(from, ArgList)
 	strcpy(buffer, ArgList[0]);
 	switch(-current_numeric)
 	{
+	case 437:
+		strcat(buffer, " (Unavailable)");
+		break;
 	case 471:
 		strcat(buffer, " (Channel is full)");
 		break;
@@ -337,7 +385,7 @@ cannot_join_channel(from, ArgList)
 		strcat(buffer, " (Bad channel mask)");
 		break;
 	}
-	put_it("%s %s", numeric_banner(), buffer);
+	put_it("%s%s", numeric_banner(), buffer);
 }
 
 
@@ -353,7 +401,7 @@ version(from, ArgList)
 		if (num_format_exists(-current_numeric))
 			display_msg(from, ArgList);
 		else
-			put_it("%s Server %s: %s %s", numeric_banner(),
+			put_it("%sServer %s: %s %s", numeric_banner(),
 				ArgList[1], ArgList[0], ArgList[2]);
 	}
 	else
@@ -362,7 +410,7 @@ version(from, ArgList)
 		if (num_format_exists(-current_numeric))
 			display_msg(from, ArgList);
 		else
-			put_it("%s Server %s: %s", numeric_banner(), ArgList[1],
+			put_it("%sServer %s: %s", numeric_banner(), ArgList[1],
 				ArgList[0]);
 	}
 }
@@ -384,7 +432,7 @@ invite(from, ArgList)
 			if (num_format_exists(-current_numeric))
 				display_msg(from, ArgList);
 			else
-				put_it("%s Inviting %s to channel %s",
+				put_it("%sInviting %s to channel %s",
 					numeric_banner(), who, channel);
 	}
 }
@@ -444,6 +492,13 @@ numbered_command(from, comm, ArgList)
 			display_msg(from, ArgList);
 		break;
 
+	case 005:	/* #define RPL_ISUPPORT         005 */
+		/* Undernet only? */
+		got_isupport(ArgList);
+		if (do_hook(current_numeric, "%s %s", from, *ArgList))
+			display_msg(from, ArgList);
+		break;
+
 /*
  * this part of ircii has been broken for most of ircd 2.7, so someday I'll
  * make it work for ircd 2.8 ...  phone..
@@ -482,9 +537,12 @@ numbered_command(from, comm, ArgList)
 		  {
 			lusers_user_cnt = user_cnt + inv_cnt;
 			/* Can't use %f in put_it() on some systems */
-			sprintf(tempstr, "%s There are %u users (%u inv) on %u servers (%3.3f clt/serv)",
-			  numeric_banner(), lusers_user_cnt, inv_cnt,
-			  server_cnt, server_cnt ? 1.0*lusers_user_cnt/server_cnt : 0.0);
+			sprintf(tempstr, "%sThere %s %u user%s (%u inv) on %u server%s (%3.3f clients/serv)",
+			  numeric_banner(), lusers_user_cnt == 1 ? "is" : "are",
+			  lusers_user_cnt, lusers_user_cnt == 1 ? "" : "s",
+			  inv_cnt,
+			  server_cnt, server_cnt == 1 ? "" : "s",
+			  server_cnt ? 1.0*lusers_user_cnt/server_cnt : 0.0);
 			smart_display(from, tempstr);
 		  } else {
 			lusers_user_cnt = -1;
@@ -502,12 +560,12 @@ numbered_command(from, comm, ArgList)
 		  oper_cnt = atoi(ArgList[0]);
 		  if (oper_cnt) {
 			/* Can't use %f in put_it() on some systems */
-			sprintf(tempstr, "%s %u IRC Operators (%3.3f clt/oper)",
-			  numeric_banner(), oper_cnt,
-			  1.0*lusers_user_cnt/oper_cnt);
+			sprintf(tempstr, "%s%u IRC Operator%s (%3.3f clients/oper)",
+			  numeric_banner(), oper_cnt, oper_cnt == 1 ? "" : "s",
+			  1.0*(lusers_user_cnt-oper_cnt)/oper_cnt);
 			smart_display(from, tempstr);
 		  } else {
-			sprintf(tempstr, "%s No IRC Operators! D'oh!",
+			sprintf(tempstr, "%sNo IRC Operators! D'oh!",
 			  numeric_banner());
 			smart_display(from, tempstr);
 		  }
@@ -523,12 +581,12 @@ numbered_command(from, comm, ArgList)
 		  chan_cnt = atoi(ArgList[0]);
 		  if (chan_cnt) {
 			/* Can't use %f in put_it() on some systems */
-			sprintf(tempstr, "%s %u channels formed (%3.3f avg users/channel)",
-			  numeric_banner(), chan_cnt,
+			sprintf(tempstr, "%s%u channel%s formed (%3.3f avg users/channel)",
+			  numeric_banner(), chan_cnt, chan_cnt == 1 ? "" : "s",
 			  1.0*lusers_user_cnt/chan_cnt);
 			smart_display(from, tempstr);
 		  } else {
-			sprintf(tempstr, "%s No channels formed!",
+			sprintf(tempstr, "%sNo channels formed!",
 			  numeric_banner());
 			smart_display(from, tempstr);
 		  }
@@ -546,8 +604,10 @@ numbered_command(from, comm, ArgList)
 		    &my_clients, &my_servers) == 2) &&
 		    (lusers_user_cnt > 0))
 		  {
-			sprintf(tempstr, "%s I have %u clients and %u servers (%3.3f%% of total)",
-			  numeric_banner(), my_clients, my_servers,
+			sprintf(tempstr, "%sI have %u client%s and %u server%s (%3.3f%% of total)",
+			  numeric_banner(), my_clients,
+			  my_clients == 1 ? "" : "s", my_servers,
+			  my_servers == 1 ? "" : "s",
 			  100.0*my_clients/lusers_user_cnt);
 			smart_display(from, tempstr);
 		  } else
@@ -881,7 +941,7 @@ numbered_command(from, comm, ArgList)
 			if (num_format_exists(221))
 				display_msg(from, ArgList);
 			else
-				put_it("%s Your user mode is \"%s\"",
+				put_it("%sYour user mode is \"%s\"",
 					numeric_banner(), ArgList[0]);
 			break;
 
@@ -894,10 +954,10 @@ numbered_command(from, comm, ArgList)
 			    from, strlen(get_server_itsname(from_server))))
 				from = NULL;
 			 if (from)
-				put_it("%s %s from (%s)", numeric_banner(),
+				put_it("%s%s from (%s)", numeric_banner(),
 					*ArgList, from);
 			 else
-				put_it("%s %s", numeric_banner(), *ArgList);
+				put_it("%s%s", numeric_banner(), *ArgList);
 			}
 			break;
 
@@ -920,14 +980,14 @@ numbered_command(from, comm, ArgList)
 				if (ArgList[2])
 				{
 					PasteArgs(ArgList, 2);
-					put_it("%s %-20s %-20s %s",
+					put_it("%s%-20s %-20s %s",
 						numeric_banner(), ArgList[0],
 						ArgList[1], ArgList[2]);
 				}
 				else
 				{
 					PasteArgs(ArgList, 1);
-					put_it("%s %-20s %s", numeric_banner(),
+					put_it("%s%-20s %s", numeric_banner(),
 						ArgList[0], ArgList[1]);
 				}
 			}
@@ -937,7 +997,7 @@ numbered_command(from, comm, ArgList)
 			if (num_format_exists(367))
 				display_msg(from, ArgList);
 			else
-				put_it("%s %s %s (%s %s)", numeric_banner(),
+				put_it("%s%s %s (%s %s)", numeric_banner(),
 					ArgList[0], ArgList[1], ArgList[2],
 					MakeTimeStamp(atoi(ArgList[3])));
 			break;
@@ -950,7 +1010,7 @@ numbered_command(from, comm, ArgList)
 				if (num_format_exists(372))
 					display_msg(from, ArgList);
 				else
-					put_it("%s %s", numeric_banner(),
+					put_it("%s%s", numeric_banner(),
 						ArgList[0]);
 			}
 			break;
@@ -963,7 +1023,7 @@ numbered_command(from, comm, ArgList)
 				if (num_format_exists(375))
 					display_msg(from, ArgList);
 				else
-					put_it("%s %s", numeric_banner(),
+					put_it("%s%s", numeric_banner(),
 						ArgList[0]);
 			}
 			break;
@@ -977,7 +1037,7 @@ numbered_command(from, comm, ArgList)
 				if (num_format_exists(376))
 					display_msg(from, ArgList);
 				else
-					put_it("%s %s", numeric_banner(),
+					put_it("%s%s", numeric_banner(),
 						ArgList[0]);
 			}
 			set_server_motd(from_server, 0);
@@ -988,7 +1048,7 @@ numbered_command(from, comm, ArgList)
 			if (num_format_exists(384))
 				display_msg(from, ArgList);
 			else
-				put_it("%s %s %s", numeric_banner(),
+				put_it("%s%s %s", numeric_banner(),
 					ArgList[0], user);
 			break;
 
@@ -1024,6 +1084,7 @@ numbered_command(from, comm, ArgList)
 				get_server_nickname(from_server));
 			break;
 
+		case 437:		/* #define ERR_UNAVAILRESOURCE  437 */
 		case 471:		/* #define ERR_CHANNELISFULL    471 */
 		case 473:		/* #define ERR_INVITEONLYCHAN   473 */
 		case 474:		/* #define ERR_BANNEDFROMCHAN   474 */
