@@ -2,15 +2,15 @@
  * parse.c: handles messages from the server.   Believe it or not.  I
  * certainly wouldn't if I were you. 
  *
- * Written By Michael Sandrof
+ * Written By Timothy Jensen
  *
- * Copyright(c) 1990 
+ * Copyright (c) 1999-2001
  *
- * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
+ * See the COPYRIGHT file, or do a /HELP COPYRIGHT 
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: parse.c,v 1.8 1999/08/19 21:00:55 toast Exp $";
+static	char	rcsid[] = "@(#)$Id: parse.c,v 1.17 2001/12/15 02:33:42 toast Exp $";
 #endif
 
 #include "irc.h"
@@ -40,10 +40,12 @@ static	char	rcsid[] = "@(#)$Id: parse.c,v 1.8 1999/08/19 21:00:55 toast Exp $";
 #include "toast.h"
 #include "format.h"
 #include "cignore.h"
+#include "rejoin.h"
 
 #define STRING_CHANNEL '+'
 #define MULTI_CHANNEL '#'
 #define LOCAL_CHANNEL '&'
+#define CHANNEL_OPS '@'
 
 #define	MAXPARA	15	/* Taken from the ircd */
 
@@ -83,6 +85,23 @@ char	*to;
 }
 
 
+/*
+ * is_channel_ops: determines if the argument is a channel and has a
+ * CHANNEL_OPS prefix to indicate a message for ops.
+ */
+int
+is_channel_ops(to)
+char	*to;
+{
+	/*
+	 * Who cares about server versions.  Isn't everyone running a 2.8
+	 * derivative or higher?  Guess I'll find out! :)
+	 */
+	return ((*to == CHANNEL_OPS) && ((to[1] == MULTI_CHANNEL) ||
+		(to[1] == LOCAL_CHANNEL)));
+}
+
+
 char	*
 PasteArgs(Args, StartPoint)
 	char	**Args;
@@ -203,8 +222,7 @@ topic(from, ArgList)
 			return;
 		message_from(ArgList[0], LOG_CRAP);
 		if (do_hook(TOPIC_LIST, "%s %s %s", from, ArgList[0], ArgList[1]))
-			say("[%s] Topic by %s on %s: %s", TimeStamp(), from,
-				ArgList[0], ArgList[1]);
+			put_it("%s", parseformat(TOPIC_FMT, from, ArgList[0], ArgList[1]));
 		message_from((char *) 0, LOG_CURRENT);
 	}
 }
@@ -261,9 +279,9 @@ wallops(from, ArgList)
 	char	*from,
 		**ArgList;
 {
-	int	flag;
+	int	flag,
+		level;
 	char	*line;
-	Window	*old_to_window;
 
 if (!from)
 		return;
@@ -285,10 +303,7 @@ if (!from)
 			level = set_lastlog_msg_level(LOG_WALLOP);
 			if (do_hook(WALLOP_LIST, "%s S %s", from, line))
 			{
-				old_to_window = to_window;
-				to_window = get_window_by_name(get_string_var(SERVER_NOTICE_WINDOW_VAR));
 				put_it("%s", parseformat(SERVER_WALLOPS_FMT, from, line));
-				to_window = old_to_window;
 			}
 			if (beep_on_level & LOG_WALLOP)
 				beep_em(1);
@@ -298,17 +313,27 @@ if (!from)
 	}
 	else
 	{
-		if (get_int_var(USER_WALLOPS_VAR))
+		if (strcmp(from, get_server_nickname(get_window_server(0)))
+			== 0)
 		{
-			if ((flag != DONT_IGNORE) && (check_flooding(from, WALLOP_FLOOD, "*", line)))
-			add_to_whois_queue(from, whois_new_wallops, "%s", line);
+			message_from(from, LOG_WALLOP);
+			level = set_lastlog_msg_level(LOG_WALLOP);
+			put_it("%s", parseformat(WALLOPS_FMT, from, line,
+				get_server_operator(get_window_server(0)) ? "*"
+				: empty_string));
+			if (beep_on_level & LOG_WALLOP)
+				beep_em(1);
+			set_lastlog_msg_level(level);
+			message_from((char *) 0, LOG_CRAP);
 		}
-		else if (strcmp(from, get_server_nickname(get_window_server(0))) != 0)
+		else if (get_int_var(USER_WALLOPS_VAR))
 		{
-			old_to_window = to_window;
-			to_window = get_window_by_name(get_string_var(SERVER_NOTICE_WINDOW_VAR));
-			put_it("%s", parseformat(WALLOPS_FMT, from, line));
-			to_window = old_to_window;
+			if ((flag != DONT_IGNORE) &&
+				(check_flooding(from, WALLOP_FLOOD, "*", line)))
+			{
+				add_to_whois_queue(from, whois_new_wallops,
+					"%s", line);
+			}
 		}
 	}
 }
@@ -567,6 +592,19 @@ p_privmsg(from, Args)
 			flood_type = PUBLIC_FLOOD;
 		}
 	}
+	else if (is_channel_ops(to))
+	{
+		to++;
+		message_from(to, LOG_MSG);
+		malloc_strcpy(&public_nick, from);
+		log_type = LOG_PUBLIC;
+		ignore_type = IGNORE_PUBLIC;
+		if (is_current_channel(to, 0))
+			list_type = OPS_LIST;
+		else
+			list_type = OPS_OTHER_LIST;
+		flood_type = PUBLIC_FLOOD;
+	}
 	else
 	{
 		message_from(from, LOG_MSG);
@@ -589,7 +627,6 @@ p_privmsg(from, Args)
 	switch (flag)
 	{
 	case IGNORED:
-/*		if ((list_type == MSG_LIST) && get_int_var(SEND_IGNORE_MSG_VAR))*/
 		if ((list_type == MSG_LIST) && do_warn(from, FromUserHost))
 			send_to_server("NOTICE %s :%s is ignoring you", from,
 					get_server_nickname(from_server));
@@ -686,6 +723,28 @@ p_privmsg(from, Args)
 			}
 			doing_privmsg = 0;
 			break;
+		case OPS_LIST:
+			if (is_cignored(to, CIGNORE_MSGS))
+				break;
+			doing_privmsg = 1;
+			if (com_do_log || com_lines)
+			{
+				com_lines++;
+				if (com_lines == 5)
+					com_lines = 0;
+				logmsg(0, "<", from, ptr, FromUserHost, 0);
+			}
+			if (no_flood && do_hook(list_type, "%s %s %s", from, 
+			    to, ptr)) {
+				int rn = get_refnum_by_channel(to);
+				if (!is_last_current_channel(rn, to)) {
+				  set_last_current_channel(rn, to);
+				  say("You are now talking to channel %s", to);
+				}
+				put_it("%s", parseformat(OPS_FMT, from, to, ptr));
+			}
+			doing_privmsg = 0;
+			break;
 		case PUBLIC_OTHER_LIST:
 			if (is_cignored(to, CIGNORE_MSGS))
 				break;
@@ -702,6 +761,22 @@ p_privmsg(from, Args)
 				put_it("%s", parseformat(PUBLIC_OTHER_FMT, from, to ,ptr));
 			doing_privmsg = 0;
 			break;
+		case OPS_OTHER_LIST:
+			if (is_cignored(to, CIGNORE_MSGS))
+				break;
+			doing_privmsg = 1;
+			if (com_do_log || com_lines)
+			{
+				com_lines++;
+				if (com_lines == 5)
+					com_lines = 0;
+				logmsg(0, "<", from, ptr, FromUserHost, 0);
+			}
+			if (no_flood && do_hook(list_type, "%s %s %s", from,
+			    to, ptr))
+				put_it("%s", parseformat(OPS_OTHER_FMT, from, to ,ptr));
+			doing_privmsg = 0;
+			break;
 		}
 		if (beep_on_level & log_type)
 			beep_em(1);
@@ -858,6 +933,7 @@ p_channel(from, ArgList)
 	int	flag;
 	char	*s;
 	int	chan_oper = 0, chan_voice = 0;
+	int	rejoin;
 
 	if (!from)
 		return;
@@ -902,12 +978,21 @@ p_channel(from, ArgList)
 	{
 		if (join)
 		{
-			add_channel(channel, from_server);
+			rejoin = is_on_rejoin_list(channel, from_server);
+			add_channel(channel, from_server, rejoin);
 			if (get_server_version(from_server) == Server2_5)
 				send_to_server("NAMES %s", channel);
 			send_to_server("MODE %s", channel);
 			add_autowho(channel);
-			set_last_current_channel(0, channel);
+			if (rejoin)
+			{
+				rejoin_kill(channel, from_server);
+				set_last_current_channel(0, NULL);
+			}
+			else
+			{
+				set_last_current_channel(0, channel);
+			}
 			send_to_server("WHO %s", channel);
 			funny_set_ignore_channel(channel);
 		}
