@@ -10,7 +10,7 @@
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: screen.c,v 1.1.1.1 1999/07/16 21:21:43 toast Exp $";
+static	char	rcsid[] = "@(#)$Id: screen.c,v 1.2 2001/09/01 01:11:21 toast Exp $";
 #endif
 
 #include "irc.h"
@@ -413,6 +413,47 @@ display_bold(flag)
 	return OFF;
 }
 
+/* display_blink: turns off and on the display blinking.  */
+static	char
+display_blink(flag)
+	int	flag;
+{
+	static	int	blink = OFF;
+
+	fflush(current_screen->fpout);
+	if (flag == blink)
+		return (flag);
+	switch (flag)
+	{
+	case ON:
+		blink = ON;
+		if (get_int_var(BLINK_VIDEO_VAR))
+			term_blink_on();
+		return (OFF);
+	case OFF:
+		blink = OFF;
+		if (get_int_var(BLINK_VIDEO_VAR))
+			term_blink_off();
+		return (ON);
+	case TOGGLE:
+		if (blink == ON)
+		{
+			blink = OFF;
+			if (get_int_var(BLINK_VIDEO_VAR))
+				term_blink_off();
+			return (ON);
+		}
+		else
+		{
+			blink = ON;
+			if (get_int_var(BLINK_VIDEO_VAR))
+				term_blink_on();
+			return (OFF);
+		}
+	}
+	return OFF;
+}
+
 /*
  * output_line prints the given string at the current screen position,
  * performing adjustments for ^_, ^B, ^V, and ^O
@@ -423,8 +464,8 @@ output_line(str, result, startpos)
 	char	**result;
 	int	startpos;
 {
-	static	int	high = OFF, bold = OFF;
-	int	rev_tog, und_tog, bld_tog, all_off;
+	static	int	high = OFF, bold = OFF, blink = OFF;
+	int	rev_tog, und_tog, bld_tog, blk_tog, all_off;
 	char	*ptr;
 	int	len;
 	int	written = 0;
@@ -435,6 +476,7 @@ output_line(str, result, startpos)
 	ptr = str;
 	display_highlight(high);
 	display_bold(bold);
+	display_blink(blink);
 	/* do processing on the string, handle inverse and bells */
 	while (*ptr)
 	{
@@ -443,6 +485,7 @@ output_line(str, result, startpos)
 		case REV_TOG:
 		case UND_TOG:
 		case BOLD_TOG:
+		case BLINK_TOG:
 		case ALL_OFF:
 			len = ptr - str;
 			written += len;
@@ -462,7 +505,7 @@ output_line(str, result, startpos)
 				fwrite(str, len, 1, current_screen->fpout);
 			  else
 				fprintf(stderr, "Tried to write a negative-length string!\n");
-			rev_tog = und_tog = bld_tog = all_off = 0;
+			rev_tog = und_tog = bld_tog = blk_tog = all_off = 0;
 			do
 			{
 				switch(*ptr)
@@ -477,13 +520,17 @@ output_line(str, result, startpos)
 				case BOLD_TOG:
 					bld_tog = 1 - bld_tog;
 					break;
+				case BLINK_TOG:
+					blk_tog = 1 - blk_tog;
+					break;
 				case ALL_OFF:
 					all_off = 1;
-					und_tog = rev_tog = bld_tog = 0;
+					und_tog = rev_tog = bld_tog = blk_tog = 0;
 					break;
 				}
 			} while ((ptr[1] == REV_TOG || ptr[1] == UND_TOG ||
-			    ptr[1] == BOLD_TOG || ptr[1] == ALL_OFF) && ptr++);
+			    ptr[1] == BOLD_TOG || ptr[1] == BLINK_TOG ||
+			    ptr[1] == ALL_OFF) && ptr++);
 			if (all_off)
 			{
 				if (!underline)
@@ -493,8 +540,10 @@ output_line(str, result, startpos)
 				}
 				display_highlight(OFF);
 				display_bold(OFF);
+				display_blink(OFF);
 				high = 0;
 				bold = 0;
+				blink = 0;
 			}
 			if (und_tog && get_int_var(UNDERLINE_VIDEO_VAR))
 			{
@@ -513,6 +562,11 @@ output_line(str, result, startpos)
 				bold = display_bold(TOGGLE);
 				bold = 1 - bold;
 			}
+			if (blk_tog)
+			{
+				blink = display_blink(TOGGLE);
+				blink = 1 - blink;
+			}
 			str = ++ptr;
 			break;
 		case '\007':
@@ -1153,6 +1207,7 @@ split_up_line(str)
 			case ALL_OFF:
 			case REV_TOG:
 			case BOLD_TOG:
+			case BLINK_TOG:
 				buffer[pos++] = *ptr;
 				nd_cnt++;
 				break;
@@ -1281,6 +1336,7 @@ add_to_window(window, str)
 		add_to_lastlog(window, str);
 		display_highlight(OFF);
 		display_bold(OFF);
+		display_blink(OFF);
 		strmcat(str, global_all_off, BIG_BUFFER_SIZE - 1);
 		logged = islogged(window);
 		for (lines = split_up_line(str); *lines; lines++)
