@@ -9,7 +9,7 @@
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: server.c,v 1.7 1999/07/26 04:37:55 toast Exp $";
+static	char	rcsid[] = "@(#)$Id: server.c,v 1.12 2001/11/15 03:20:43 toast Exp $";
 #endif
 
 #include "irc.h"
@@ -107,6 +107,20 @@ clear_autowhos(i)
 	}
 }
 
+static	void
+clear_rejoins(i)
+	int	i;
+{
+	RejoinChan	*next;
+
+	while (server_list[i].rejoins) {
+		next = server_list[i].rejoins->next;
+		new_free(&server_list[i].rejoins->channel);
+		new_free(&server_list[i].rejoins);
+		server_list[i].rejoins = next;
+	}
+}
+
 /*
  * Why this as well as MyHostAddr? Well, if your host is a gateway, this
  * will be set to the address for your host which is nearest to everybody
@@ -164,13 +178,12 @@ close_server(server_index, message)
 			clear_channel_list(i);
 			from_server = old_server;
 		}
-#ifndef REJOIN_OPER
-		server_list[i].operator = 0;
-#endif /* EHUD */
+		/*server_list[i].operator = 0;*/
 		server_list[i].connected = 0;
 		clear_spings(i);
 		clear_autoops(i);
 		clear_autowhos(i);
+		clear_rejoins(i);
 		server_list[i].buffer = (char *) 0;
 		if (-1 != server_list[i].write)
 		{
@@ -440,14 +453,13 @@ add_to_server_list(server, port, password, nick, overwrite)
 		server_list[from_server].away = (char *) 0;
 		server_list[from_server].version_string = (char *) 0;
 		server_list[from_server].operator = 0;
-#ifdef REJOIN_OPER
 	   	server_list[from_server].op_nick = (char *) 0;
                 server_list[from_server].op_pwd = (char *) 0;
-#endif /* EHUD */
 		server_list[from_server].read = -1;
 		server_list[from_server].write = -1;
 		server_list[from_server].pid = -1;
 		server_list[from_server].version = 0;
+		server_list[from_server].options = SOptions_None;
 		server_list[from_server].whois = 0;
 		server_list[from_server].flags = SERVER_2_6_2;
 		server_list[from_server].nickname = (char *) 0;
@@ -482,6 +494,7 @@ add_to_server_list(server, port, password, nick, overwrite)
 		server_list[from_server].OpTimers = (AutoOp_Timer *) 0;
 		server_list[from_server].spingstuff = (SpingStuff *) 0;
 		server_list[from_server].autowho = (AutoWho *) 0;
+		server_list[from_server].rejoins = (RejoinChan *) 0;
 	}
 	else
 	{
@@ -521,12 +534,10 @@ remove_from_server_list(i)
 		new_free(&server_list[i].password);
 	if (server_list[i].away)
 		new_free(&server_list[i].away);
-#ifdef REJOIN_OPER
         if (server_list[i].op_nick)
                 new_free(&server_list[i].op_nick);
         if (server_list[i].op_pwd)
                 new_free(&server_list[i].op_pwd);
-#endif /* EHUD */
 	if (server_list[i].version_string)
 		new_free(&server_list[i].version_string);
 	if (server_list[i].nickname)
@@ -724,9 +735,7 @@ connect_to_server_direct(server_name, port)
 	else
 		server_list[from_server].read = new_des;
 	server_list[from_server].local_addr.s_addr = localaddr.sin_addr.s_addr;
-#ifndef REJOIN_OPER
-	server_list[from_server].operator = 0;
-#endif /* EHUD */
+	/*server_list[from_server].operator = 0;*/
 	return (0);
 }
 
@@ -861,7 +870,7 @@ connect_to_server(server_name, port, c_server)
 
 	message_from((char *) 0, LOG_CURRENT);
 	server_index = find_in_server_list(server_name, port);
-	attempting_to_connect++;
+	attempting_to_connect = 1;	/* actually lets just set it */
 	if (server_index == -1 || server_list[server_index].read == -1)
 	{
 		if (port == -1)
@@ -878,7 +887,7 @@ connect_to_server(server_name, port, c_server)
 			server_index = connect_to_server_direct(server_name, port);
 		if (server_index)
 		{
-			attempting_to_connect--;
+			attempting_to_connect = 0;	/* just clear it */
 			return -1;
 		}
 		if ((c_server != -1) && (c_server != from_server))
@@ -1374,6 +1383,33 @@ get_server_version(server_index)
 	return (server_list[server_index].version);
 }
 
+/*
+ * set_server_options: Sets the server options for the given server index.
+ * See server.h for values and their meanings.
+ */
+void
+set_server_options(server_index, options)
+	int	server_index;
+	int	options;
+{
+	if (server_index == -1)
+		server_index = primary_server;
+	server_list[server_index].options |= options;
+}
+
+/*
+ * get_server_options: returns the server options value for the given server
+ * index.
+ */
+int
+get_server_options(server_index)
+	int	server_index;
+{
+	if (server_index == -1)
+		server_index = primary_server;
+	return (server_list[server_index].options);
+}
+
 /* get_server_name: returns the name for the given server index */
 char	*
 get_server_name(server_index)
@@ -1529,20 +1565,20 @@ get_server_operator(server_index)
 	return (server_list[server_index].operator);
 }
 
-#ifdef REJOIN_OPER
 char 
 *get_server_operator_pwd(server_index)
 	int	server_index;
 {	
 	return (server_list[server_index].op_pwd);
 }
+
 char
 *get_server_operator_nick(server_index)
         int     server_index;
 {
         return (server_list[server_index].op_nick);
 }
-#endif /* EHUD */
+
 /*
  * set_server_operator: If flag is non-zero, marks the user as having op
  * privs on the given server.  
@@ -1573,7 +1609,7 @@ set_server_nickname(server_index, nick)
 	}
 	update_all_status();
 }
-#ifdef REJOIN_OPER
+
 void
 set_server_operator_pwd(server_index, pwd)
 	int	server_index;
@@ -1585,6 +1621,38 @@ set_server_operator_pwd(server_index, pwd)
 	}
 	update_all_status();
 }
+
+void
+clear_server_operator_pwd(server_index)
+	int	server_index;
+{
+	int len;
+
+        if (server_index != -1)
+        {
+		if (server_list[server_index].op_pwd)
+		{
+			len = strlen(server_list[server_index].op_pwd);
+			memset(server_list[server_index].op_pwd, 0, len);
+			memset(server_list[server_index].op_pwd, 255, len);
+			memset(server_list[server_index].op_pwd, 0, len);
+			new_free(&(server_list[server_index].op_pwd));
+		}
+	}
+	update_all_status();
+}
+
+void
+clear_all_server_operator_pwd(void)
+{
+	int server_index;
+
+        for (server_index = 0; server_index < number_of_servers; server_index++)
+        {
+		clear_server_operator_pwd(server_index);
+	}
+}
+
 void
 set_server_operator_nick(server_index, nick)
         int     server_index;
@@ -1597,7 +1665,6 @@ set_server_operator_nick(server_index, nick)
         update_all_status();
 }
 
-#endif /* EHUD */
 void
 set_server_motd(server_index, flag)
 	int	server_index;
@@ -1812,3 +1879,24 @@ done:
 	clean_whois_queue();
 	window_check_servers();
 }
+
+
+/*
+:austin.tx.us.undernet.org 005 ToastTEST SILENCE=15 WHOX WALLCHOPS USERIP CPRIVMSG CNOTICE MODES=6 MAXCHANNELS=10 MAXBANS=30 NICKLEN=9 TOPICLEN=160 KICKLEN=160 CHANTYPES=+#& :are supported by this server
+:austin.tx.us.undernet.org 005 ToastTEST  PREFIX=(ov)@+ CHANMODES=b,k,l,imnpst CHARSET=rfc1459 :are supported by this server
+*/
+void
+got_isupport(args)
+	char	**args;
+{
+	int	i;
+
+	for (i = 0; args[i]; i++)
+	{
+		if (!my_stricmp(args[i], "WALLCHOPS"))
+		{
+			server_list[from_server].options |= SOptions_WallChops;
+		}
+	}
+}
+
