@@ -9,7 +9,7 @@
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: status.c,v 1.3 1999/08/23 01:34:06 toast Exp $";
+static	char	rcsid[] = "@(#)$Id: status.c,v 1.5 2001/12/09 05:28:31 toast Exp $";
 #endif
 
 #include "irc.h"
@@ -633,7 +633,7 @@ make_status(window)
 			{
 				RJustifyPos = i;
 			}
-			else if (buffer[i] != REV_TOG && buffer[i] != UND_TOG && buffer[i] != ALL_OFF && buffer[i] != BOLD_TOG)
+			else if (buffer[i] != REV_TOG && buffer[i] != UND_TOG && buffer[i] != ALL_OFF && buffer[i] != BOLD_TOG && buffer[i] != BLINK_TOG)
 			{
 				if (RealPosition == CO)
 				{
@@ -679,7 +679,7 @@ make_status(window)
 		buffer[len] = ALL_OFF;
 		buffer[len+1] = (char) 0;
 
-		status_make_printable(buffer, len);
+		status_make_printable((unsigned char *)buffer, len);
 
 	/*
 	 * Thanks to Max Bell (mbell@cie.uoregon.edu) for info about TVI
@@ -692,7 +692,7 @@ make_status(window)
 			{
 				if (buffer[i] != window->status_line[i])
 					break;
-				if (buffer[i] != REV_TOG && buffer[i] != UND_TOG && buffer[i] != ALL_OFF && buffer[i] != BOLD_TOG)
+				if (buffer[i] != REV_TOG && buffer[i] != UND_TOG && buffer[i] != ALL_OFF && buffer[i] != BOLD_TOG && buffer[i] != BLINK_TOG)
 					RealPosition++;
 			}
 		}
@@ -1300,6 +1300,7 @@ status_make_printable(str, n)
 			case ALL_OFF:
 			case REV_TOG:
 			case BOLD_TOG:
+			case BLINK_TOG:
 				buffer[pos++] = *s;
 				break;
 			default:
@@ -1319,5 +1320,5 @@ status_make_printable(str, n)
 			buffer[pos++] = *s;
 	}
 	buffer[pos] = '\0';
-	strncpy(str, buffer, pos);
+	strncpy((char *)str, buffer, pos);
 }
