@@ -12,7 +12,7 @@
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: term.c,v 1.2 1999/07/25 17:08:17 toast Exp $";
+static	char	rcsid[] = "@(#)$Id: term.c,v 1.4 2001/11/13 22:11:26 toast Exp $";
 #endif
 
 #include "irc.h"
@@ -193,13 +193,16 @@ char	*CM,
 	*SE,
 	*US,
 	*UE,
+	*MB,
 	*MD,
 	*ME,
 	*SF,
 	*SR,
 	*ND,
 	*LE,
-	*BL;
+	*BL,
+	*AF,
+	*AB;
 int	CO = 79,
 	LI = 24,
 	SG;
@@ -600,15 +603,39 @@ term_init()
 		US = empty_string;
 		UE = empty_string;
 	}
+
+	/* Start blinking */
+	MB = tgetstr("mb", &ptr);
+	if (MB == (char *) 0)
+	{
+		MB = empty_string;
+	}
+
+	/* Start bold mode */
 	MD = tgetstr("md", &ptr);
+
+	/* End all mode like so, us, mb, md and mr */
 	ME = tgetstr("me", &ptr);
+
 	if ((MD == (char *) 0) || (ME == (char *) 0))
 	{
+		MB = empty_string;
 		MD = empty_string;
 		ME = empty_string;
 	}
+
+	/* Bell */
 	if ((BL = tgetstr("bl", &ptr)) == (char *) 0)
 		BL = "\007";
+
+	/* ANSI foreground color */
+	if ((AF = tgetstr("AF", &ptr)) == (char *) 0)
+		AF = empty_string;
+
+	/* ANSI background color */
+	if ((AB = tgetstr("AB", &ptr)) == (char *) 0)
+		AB = empty_string;
+
 #endif /* STTY_ONLY */
 }
 
