@@ -3,13 +3,13 @@
  *
  * Written By Timothy Jensen
  *
- * Copyright(c) 1999 
+ * Copyright (c) 1999-2001
  *
- * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
+ * See the COPYRIGHT file, or do a /HELP COPYRIGHT 
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: toast.c,v 1.14 1999/08/05 17:53:00 toast Exp $";
+static	char	rcsid[] = "@(#)$Id: toast.c,v 1.27 2001/12/08 16:50:31 toast Exp $";
 #endif
 
 #include "irc.h"
@@ -31,6 +31,9 @@ static	char	rcsid[] = "@(#)$Id: toast.c,v 1.14 1999/08/05 17:53:00 toast Exp $";
 #include "input.h"
 #include "names.h"
 #include "server.h"
+#include "output.h"
+#include "numbers.h"
+#include "comstud.h"
 
 char	dummy[1024];	/* Stupid temporary string */
 
@@ -53,6 +56,8 @@ char	*oops_nick = (char *) 0,
 	*oops_line = (char *) 0,
 	*oops_command = (char *) 0;
 
+int	setting_nolog = 0;
+
 typedef struct away_struct
 {
   char *nick;
@@ -61,29 +66,41 @@ typedef struct away_struct
 
 Away	last_away[10] = {{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}};
 
+char *cs_channel;
+int cs_total, cs_ops, cs_here, cs_ircops, cs_hops;
+
+
 /*
  * TimeStamp -- makes a nice stampstamp in the following format:
  *
  * [15-Jul:11:36 mjr] asdf
  *
  */
-
-char *TimeStamp()
+char *
+TimeStamp(void)
 {
   static char ts_holder[64];
   struct tm *time_val;
   time_t t;
+  char *ts;
 
   t = time(0);
   time_val = localtime(&t);
+  ts = get_string_var(TIMESTAMP_VAR);
 
-  strftime(ts_holder, 64, get_string_var(TIMESTAMP_VAR), time_val);
-  ts_holder[63] = 0;
+  if (ts != NULL) {
+    strftime(ts_holder, 64, ts, time_val);
+    ts_holder[63] = 0;
+  } else
+    ts_holder[0] = 0;
 
   return (char *)&ts_holder;
 }
 
-char *MakeTimeStamp(long thetime)
+
+char *
+MakeTimeStamp(thetime)
+	long	thetime;
 {
   static char ts_holder[64];
   struct tm *time_val;
@@ -96,7 +113,9 @@ char *MakeTimeStamp(long thetime)
   return (char *)&ts_holder;
 }
 
-char *long_to_time(input)
+
+char *
+long_to_time(input)
         char    *input;
 {
         char    result[64];
@@ -117,7 +136,9 @@ char *long_to_time(input)
         }
 }
 
-void channel_created(from, ArgList)
+
+void
+channel_created(from, ArgList)
 	char	*from,
 		**ArgList;
 {
@@ -129,11 +150,20 @@ void channel_created(from, ArgList)
 		return;
 	when = ArgList[1];
 	message_from(channel, LOG_CRAP);
-	put_it("%s Channel %s created on %s", numeric_banner(), channel,
-		long_to_time(when));
+	if (num_format_exists(-current_numeric))
+	{
+		put_it("%s", numericformat(-current_numeric, from,
+			numeric_banner(), ArgList));
+	}
+	else
+	{
+		put_it("%sChannel %s created on %s", numeric_banner(), channel,
+			long_to_time(when));
+	}
 }
 
-void channel_topictime(from, ArgList)
+void
+channel_topictime(from, ArgList)
         char    *from,
                 **ArgList;
 {
@@ -147,12 +177,21 @@ void channel_topictime(from, ArgList)
 	nick = ArgList[1];
         when = ArgList[2];
         message_from(channel, LOG_CRAP);
-        put_it("%s Topic for %s set by %s on %s", numeric_banner(), channel,
-                nick, long_to_time(when));
+	if (num_format_exists(-current_numeric))
+	{
+		put_it("%s", numericformat(-current_numeric, from,
+			numeric_banner(), ArgList));
+	}
+	else
+	{
+        	put_it("%sTopic for %s set by %s on %s", numeric_banner(),
+			channel, nick, long_to_time(when));
+	}
 }
 
 
-int Do_Not_Log(from)
+int
+Do_Not_Log(from)
 	char	*from;
 {
 	char	*NoLogList;
@@ -188,7 +227,8 @@ int Do_Not_Log(from)
  *
  */
 
-void nolog(command, args)
+void
+nolog(command, args)
 	char	*command,
 		*args;
 {
@@ -205,8 +245,8 @@ void nolog(command, args)
 	}
 
 	/* Go through each nick specified afer command */
-	while (NL_nick = next_arg(args, &args)) {
-		
+	while (NULL != (NL_nick = next_arg(args, &args)))
+	{
 		int	flag;
 
 		NoLogList = get_string_var(NO_LOG_VAR);
@@ -214,8 +254,8 @@ void nolog(command, args)
 		dummy[0] = 0;
 		flag = 0;
 		/* Now check if this nick is already in the list */
-		while (NLL_nick = next_arg(NoLogList, &NoLogList)) {
-
+		while (NULL != (NLL_nick = next_arg(NoLogList, &NoLogList)))
+		{
 			int	len;
 
 			if (strlen(NL_nick) > strlen(NLL_nick))
@@ -226,6 +266,7 @@ void nolog(command, args)
 			if (!my_strnicmp(NL_nick, NLL_nick, len)) {
 				flag = 1;
 				say("Removing from NO_LOG: %s", NL_nick);
+				logremovenolog(NL_nick);
 			} else {
 				strmcat(&dummy, NLL_nick, 1024);
 				strmcat(&dummy, " ", 1024);
@@ -234,13 +275,24 @@ void nolog(command, args)
 		if (!flag) {
 			strmcat(&dummy, NL_nick, 1024);
 			say("Adding to NO_LOG: %s", NL_nick);
+			logaddnolog(NL_nick);
 		}
-
+		setting_nolog = 1;
 		set_string_var(NO_LOG_VAR, &dummy);
+		setting_nolog = 0;
 	}
 }
 
-void doleave(command, args)
+void
+set_nolog(data)
+	char	*data;
+{
+	if (!setting_nolog)
+		lognolog(data);
+}
+
+void
+doleave(command, args)
 	char	*command,
 		*args;
 {
@@ -257,7 +309,8 @@ void doleave(command, args)
 		send_to_server("PART %s", args);
 }
 
-void dowho(command, args, subargs)
+void
+dowho(command, args, subargs)
 	char	*command,
 		*args,
 		*subargs;
@@ -268,7 +321,8 @@ void dowho(command, args, subargs)
 		who("WHO", args, subargs);
 }
 
-void dowii(command, args, subargs)
+void
+dowii(command, args, subargs)
 	char	*command,
 		*args,
 		*subargs;
@@ -281,7 +335,8 @@ void dowii(command, args, subargs)
 	whois("WHOIS", dummy, subargs);
 }
 
-void dotopic(command, args, subargs)
+void
+dotopic(command, args, subargs)
 	char	*command,
 		*args,
 		*subargs;
@@ -294,7 +349,8 @@ void dotopic(command, args, subargs)
 	send_topic("TOPIC", dummy, subargs);
 }
 
-void domode(command, args, subargs)
+void
+domode(command, args, subargs)
 	char	*command,
 		*args,
 		*subargs;
@@ -303,6 +359,7 @@ void domode(command, args, subargs)
 	send_channel_com("MODE", dummy, subargs);
 }
 
+
 /*
  * KLINE - Why didn't anyone ever add this command before?
  *
@@ -311,7 +368,8 @@ void domode(command, args, subargs)
  *  /KLINE <duration> <target> <reason>     (also /TKLINE)
  *
  */
-void dokline(command, args)
+void
+dokline(command, args)
 	char	*command,
 		*args;
 {
@@ -320,7 +378,7 @@ void dokline(command, args)
 		*comma;
 	int	tkline,
 		dline,
-		dur;
+		dur = 0;
 
 	duration = next_arg(args, &args);
 
@@ -333,7 +391,7 @@ void dokline(command, args)
 
 	if (!dline && tkline) {
 		if (!duration || !is_number(duration)) {
-			say("Uh...Meathead, specify a valid temporary kline duration!");
+			say("Invalid temporary kline duration!");
 			return;
 		} else
 			dur = atoi(duration);
@@ -366,7 +424,8 @@ void dokline(command, args)
 	}
 }
 
-void unkline(command, args)
+void
+unkline(command, args)
 	char	*command,
 		*args;
 {
@@ -378,39 +437,44 @@ void unkline(command, args)
 	send_to_server("UNKLINE %s", args);
 }
 
-void vercmd(command, args)
+void
+vercmd(command, args)
 	char	*command,
 		*args;
 {
 	if (args && *args)
 		sprintf(dummy, "%s VERSION", args);
 	else
-		sprintf(dummy, "* VERSION", args);
+		sprintf(dummy, "* VERSION");
 	ctcp(command, dummy, empty_string);
 }
 
-void raw_type(text)
+void
+raw_type(text)
 	char	*text;
 {
 	int	i = 0;
 
 	while (text && (text[i] != 0))
-		input_add_character(text[i++]);
+		input_add_character(text[i++], NULL);
 }
 
-void auto_msg()
+void
+auto_msg(void)
 {
 	char	*msgnick;
 	char	*oldline = (char *) 0;
-	char	*oldcmd,*oldnick,*oldrest;
-	int	i;
+	char	*oldcmd, *oldnick, *oldrest;
+	char	*cmdchars;
 
 	if (!auto_msg_list_size) {
 		return;
 	}
 	
 	/* Get the "/MSG" first */
-	sprintf(dummy, "%sMSG", alias_cmdchar());
+	if ((cmdchars = get_string_var(CMDCHARS_VAR)) == (char *) 0)
+		cmdchars = DEFAULT_CMDCHARS;
+	sprintf(dummy, "%cMSG", cmdchars[0]);
 
 	malloc_strcpy(&oldline, get_input());
 	oldrest = oldline;
@@ -433,7 +497,8 @@ void auto_msg()
 	new_free(&oldline);
 }
 
-void remnick(anick)
+void
+remnick(anick)
 	char	*anick;
 {
 	int	i,j;
@@ -449,10 +514,11 @@ void remnick(anick)
 	}
 }
 
-void addnick(anick)
+void
+addnick(anick)
 	char	*anick;
 {
-	int	i,j;
+	int	i;
 	char	*temp = (char *) NULL;
 
 	malloc_strcpy(&temp, anick);
@@ -467,7 +533,8 @@ void addnick(anick)
 	auto_msg_list_pos = 0;
 }
 
-void adddccnick(anick)
+void
+adddccnick(anick)
 	char	*anick;
 {
     /*
@@ -478,7 +545,8 @@ void adddccnick(anick)
 	addnick(dummy);
 }
 
-void do_new_window()
+void
+do_new_window()
 {
 	in_window_command = 1;  
 	message_from((char *) 0, LOG_CURRENT); 
@@ -489,7 +557,8 @@ void do_new_window()
 	cursor_to_input();
 }
 
-void do_kill_window()
+void
+do_kill_window()
 {
 	in_window_command = 1;  
 	message_from((char *) 0, LOG_CURRENT); 
@@ -500,7 +569,8 @@ void do_kill_window()
 	cursor_to_input();
 }
 
-void do_list_windows()
+void
+do_list_windows()
 {
 	in_window_command = 1;  
 	message_from((char *) 0, LOG_CURRENT); 
@@ -511,7 +581,8 @@ void do_list_windows()
 	cursor_to_input();
 }
 
-void do_grow_window()
+void
+do_grow_window()
 {
 	in_window_command = 1;  
 	message_from((char *) 0, LOG_CURRENT); 
@@ -522,7 +593,8 @@ void do_grow_window()
 	cursor_to_input();
 }
 
-void do_shrink_window()
+void
+do_shrink_window()
 {
 	in_window_command = 1;  
 	message_from((char *) 0, LOG_CURRENT); 
@@ -533,7 +605,8 @@ void do_shrink_window()
 	cursor_to_input();
 }
 
-void do_hide_window()
+void
+do_hide_window()
 {
 	in_window_command = 1;  
 	message_from((char *) 0, LOG_CURRENT); 
@@ -544,7 +617,8 @@ void do_hide_window()
 	cursor_to_input();
 }
 
-void do_opervision()
+void
+do_opervision(void)
 {
 	Window	*window;
 
@@ -556,7 +630,7 @@ void do_opervision()
 		window = new_window();
 		malloc_strcpy(&window->name, get_string_var(SERVER_NOTICE_WINDOW_VAR));
 		window->update |= UPDATE_STATUS;
-		window->window_level = parse_lastlog_level("NONE");
+		window->window_level = LOG_WALLOP | LOG_SNOTE | LOG_OPNOTE;
 		revamp_window_levels(window);
 	} else
 		delete_window(window);
@@ -566,19 +640,22 @@ void do_opervision()
 	cursor_to_input();
 }
 
-void do_pageup()
+void
+do_pageup()
 {
 	page_updown_key = 1;
 	scrollback_backwards();
 }
 
-void do_pagedown()
+void
+do_pagedown()
 {
 	page_updown_key = 1;
 	scrollback_forwards();
 }
 
-void handle_findu(class, nuh)
+void
+handle_findu(class, nuh)
 	int	class;
 	char	*nuh;
 {
@@ -605,9 +682,10 @@ void handle_findu(class, nuh)
 	}
 }
 
-void findu(command, args)
-char *command;
-char *args;
+void
+findu(command, args)
+	char	*command,
+		*args;
 {
 	int len;
 	char *thisarg;
@@ -619,13 +697,13 @@ char *args;
 
 	toast_arg_int = -1;
 	find_type = FIND_NONE;
-	while (thisarg = my_next_arg(&args))
+	while (NULL != (thisarg = my_next_arg(&args)))
 	{
 		if (*thisarg == '-') {
 			thisarg++;
 			len = strlen(thisarg);
 			if (!my_strnicmp(thisarg, "CLASS", (len>2)?len:2)) {
-				if (thisarg = my_next_arg(&args))
+				if (NULL != (thisarg = my_next_arg(&args)))
 					toast_arg_int = atoi(thisarg);
 				else
 				{
@@ -662,7 +740,8 @@ char *args;
 :irc2.blackened.com 216 toastwank K plasma.engr.Arizona.EDU * *sphere* PERM this script only is used for abuse and annoyance. (1997/11/17 18.03)
 :irc2.blackened.com 219 toastwank K :End of /STATS report
 */
-void handle_findk(parms)
+void
+handle_findk(parms)
 	char	**parms;
 {
 	int	i;
@@ -692,9 +771,10 @@ void handle_findk(parms)
 	}
 }
 
-void findk(command, args)
-char *command;
-char *args;
+void
+findk(command, args)
+	char	*command,
+		*args;
 {
 	char *pattern = (char *) 0,
 	     *server = (char *) 0;
@@ -707,8 +787,10 @@ char *args;
 	}
 	
 	find_type = FIND_NONE;
-	while (thisarg = my_next_arg(&args)) {
-		if (*thisarg == '-') {
+	while (NULL != (thisarg = my_next_arg(&args)))
+	{
+		if (*thisarg == '-')
+		{
 			thisarg++;
 			len = strlen(thisarg);
 			if (!my_strnicmp(thisarg, "COUNT", (len>1)?len:1)) {
@@ -744,9 +826,10 @@ char *args;
 	}
 }
 
-void bancount(command, args)
-char *command;
-char *args;
+void
+bancount(command, args)
+	char	*command,
+		*args;
 {
 	char *tmp_chname = 0;
 	if (toast_action) {
@@ -766,12 +849,10 @@ char *args;
 		say("You aren't in a channel!");
 }
 
-char *cs_channel;
-int cs_total, cs_ops, cs_here, cs_ircops, cs_hops;
-
-void handle_cstat(stat, hops)
-char *stat;
-int hops;
+void
+handle_cstat(stat, hops)
+	char	*stat;
+	int	hops;
 {
 	if (stat) {
 		cs_total++;
@@ -795,9 +876,10 @@ int hops;
 	}
 }
 
-void cstat(command, args)
-char *command;
-char *args;
+void
+cstat(command, args)
+	char	*command,
+		*args;
 {
 	if (toast_action) {
 		say("Another operation in progress. Please wait.");
@@ -818,13 +900,13 @@ char *args;
 		say("You aren't in a channel!");
 }
 
-void userhost_notify(stuff, nick, args)
-WhoisStuff *stuff;
-char *nick;
-char *args;
+void
+userhost_notify(stuff, nick, args)
+	WhoisStuff *stuff;
+	char	*nick,
+		*args;
 {
         char junk[512];
-        char *temp;
 
         if (!stuff || !stuff->nick || !nick || my_stricmp(stuff->nick, nick))
                 return;
@@ -833,8 +915,9 @@ char *args;
 	put_it("%s", parseformat(NOTIFY_SIGNON_USERHOST_FMT, nick));
 }
 
-int handle_sping(from)
-char *from;
+int
+handle_sping(from)
+	char *from;
 {
 	int result;
 	SpingStuff *stuff, *laststuff;
@@ -869,8 +952,9 @@ char *from;
 	return 1;
 }
 
-void handle_sping_nosuchserver(from)
-char *from;
+void
+handle_sping_nosuchserver(from)
+	char	*from;
 {
 	SpingStuff *stuff, *laststuff;
 
@@ -894,9 +978,10 @@ char *from;
 	}
 }
 
-void sping(command, args)
-char *command;
-char *args;
+void
+sping(command, args)
+	char	*command,
+		*args;
 {
 	int result;
 	SpingStuff *stuff,*tempstuff;
@@ -945,7 +1030,8 @@ char *args;
 	}
 }
 
-void show_banner()
+void
+show_banner(void)
 {
 	put_it(":");
 	put_it(":       \026\002    ____  _        _    ____ _  _______ _   _ _____ ____     \002\026");
@@ -954,26 +1040,33 @@ void show_banner()
 	put_it(":       \026\002   | |_) | |___ / ___ \\ |___| . \\| |___| |\\  | |___| |_| |   \002\026");
 	put_it(":       \026\002   |____/|_____/_/   \\_\\____|_|\\_\\_____|_| \\_|_____|____/    \002\026");
 	put_it(":       \026\002                                                             \002\026");
-	put_it(":");
-	put_it(":           an ircii-2.8.2 based client by Timothy \"Toast\" Jensen");
+	put_it(":           ");
+	put_it(":           Blackened IRC version %s, an ircii-2.8.2 based client", irc_version);
+	put_it(":           Copyright (C) 2001  Timothy L. Jensen");
+	put_it(":             ");
 	put_it(":             Offical Website: \002http://www.blackened.com/blackened/\002");
-	put_it(":");
+	put_it(":             ");
+	put_it(": Blackened IRC is free software with ABSOLUTELY NO WARRANTY.");
+	put_it(": For details type `/help warranty'.");
+	put_it(":             ");
 }
 
-void umode(command, args)
-char *command;
-char *args;
+void
+umode(command, args)
+	char	*command,
+		*args;
 {
   if (args) {
     send_to_server("MODE %s %s", get_server_nickname(curr_scr_win->server), args);
   }
 }
 
-void handle_whokill(nick, user, host, server)
-char 	*nick,
-	*user,
-	*host,
-	*server;
+void
+handle_whokill(nick, user, host, server)
+	char 	*nick,
+		*user,
+		*host,
+		*server;
 {
 	sprintf(dummy, "%s@%s", user, host);
 	if (!nick)
@@ -988,9 +1081,10 @@ char 	*nick,
 		*reason ? reason : get_reason(), cluster(dummy), server);	
 }
 
-void add_to_away(nick, message)
-char	*nick,
-	*message;
+void
+add_to_away(nick, message)
+	char	*nick,
+		*message;
 {
 	int i;
  	new_free(&last_away[9].nick);   
@@ -1004,9 +1098,10 @@ char	*nick,
 	malloc_strcpy(&last_away[0].message, message);
 }
 
-int seen_away_msg(nick, message)
-char	*nick,
-	*message;
+int
+seen_away_msg(nick, message)
+	char	*nick,
+		*message;
 {
 	int i;
 	for (i = 0; i < 10; i++) {
@@ -1021,9 +1116,9 @@ char	*nick,
 
 char *
 function_randread(input)
-char	*input;
+	char	*input;
 {
-  int count, min, max, i;
+  int count, min, i;
   FILE *bleah;
   char buffer[1024];
   char *result = (char *) 0;
@@ -1081,13 +1176,13 @@ SmallTS(when)
 
   tm = localtime(when);
 
-  strftime(ts, 128, "%d %b %Y %T", tm);
+  strftime(ts, 128, "%d %b %Y %T %Z", tm);
 
   return (char *)&ts;
 }
 
 void
-awaylistcmd()
+awaylistcmd(void)
 {
 	int i;
 	say("Away list:");
@@ -1272,7 +1367,8 @@ BanTimer()
 :plasma.engr.arizona.edu 225 lijasdf D 1.1.1.* foo [Toast@OOMon]
 :plasma.engr.arizona.edu 219 lijasdf d :End of /STATS report
 */
-void handle_findd(parms)
+void
+handle_findd(parms)
 	char	**parms;
 {
 	int	i;
@@ -1298,9 +1394,10 @@ void handle_findd(parms)
 	}
 }
 
-void findd(command, args)
-char *command;
-char *args;
+void
+findd(command, args)
+	char	*command;
+	char	*args;
 {
 	char *pattern,
 	     *server;
@@ -1376,10 +1473,11 @@ char *args;
 
 void show_tempbans()
 {
-	char	format[40];
+	char	format[40], timestr[15];
 	BanList	*tmp;
 	time_t	now;
 	int	cwidth;
+	int	left;
 
 	now = time(NULL);
 
@@ -1390,13 +1488,15 @@ void show_tempbans()
 		return;
 	}
 	cwidth = get_int_var(CHANNEL_NAME_WIDTH_VAR);
-	sprintf(format, "%%-5s %%-%ds %%s", (unsigned char) cwidth);
+	sprintf(format, "%%-6s %%-%ds %%s", (unsigned char) cwidth);
 	say(format, "Time", "Channel", "Mask");
-	sprintf(format, "%%-5d %%-%ds %%s", (unsigned char) cwidth);
 	while (tmp) {
-		if (tmp->server == from_server)
-			say(format, (tmp->timeout-now)/60, tmp->channel,
-				tmp->mask);
+		if (tmp->server == from_server) {
+			left = tmp->timeout - now;
+			sprintf(timestr, left < 60 ? "%ds" : "%dm",
+				left < 60 ? left : left/60);
+			say(format, timestr, tmp->channel, tmp->mask);
+		}
 		
 		tmp = tmp->next;
 	}
@@ -1669,3 +1769,26 @@ char *args;
 	if (!count)
 		say("No matches for massdevoice of %s on %s", spec, to);
 }
+
+void
+toasttest(command, args)
+char	*command;
+char	*args;
+{
+	char *dump1 = NULL;
+	char *dump2 = NULL;
+	char *who, *who_uh, *server, *dead, *reason;
+
+	malloc_strcpy(&dump1, "*** Notice -- Received KILL message for imanoste. From cs-pub.bu.edu Path: irc1.portal.com[156.151.6.50]!irc2.ece.uiuc.edu[128.174.64.10]!*.bu.edu[cs-pub.bu.edu]!cs-pub.bu.edu (imanoste <- irc2.ece.uiuc.edu)");
+	/*malloc_strcpy(&dump2, "*** Notice -- Received KILL message for killtest. From Toast Path: plasma.powered!toast!Toast (testing)");*/
+	malloc_strcpy(&dump2, "*** Notice -- Received KILL message for killtest. From Toast Path: lalalala!liwjer!lijs!butt.com!plasma.powered!Toast (testing)");
+
+	split_kill(dump1, &who, &who_uh, &server, &dead, &reason);
+	say("dump1 who: %s  who_uh: %s  server: %s  dead: %s  reason: %s",
+		who, who_uh, server, dead, reason);
+
+	split_kill(dump2, &who, &who_uh, &server, &dead, &reason);
+	say("dump2 who: %s  who_uh: %s  server: %s  dead: %s  reason: %s",
+		who, who_uh, server, dead, reason);
+}
+
