@@ -10,7 +10,7 @@
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: vars.c,v 1.7 1999/08/20 22:05:35 toast Exp $";
+static	char	rcsid[] = "@(#)$Id: vars.c,v 1.20 2001/11/29 22:10:19 toast Exp $";
 #endif
 
 #include "irc.h"
@@ -29,7 +29,9 @@ static	char	rcsid[] = "@(#)$Id: vars.c,v 1.7 1999/08/20 22:05:35 toast Exp $";
 #include "translat.h"
 #include "term.h"
 #include "output.h"
+#include "server.h"
 #include "comstud.h"
+#include "toast.h"
 
 /* IrcVariable: structure for each variable in the variable table */
 typedef struct
@@ -68,6 +70,8 @@ static	void	exec_warning();
 static	void	input_warning();
 static	void	eight_bit_characters();
 static	void	set_realname _((char *));
+static	void	set_auto_reoper _((int));
+static	void	set_ops_method_var _((char *));
 
 /*
  * irc_variable: all the irc variables used.  Note that the integer and
@@ -77,18 +81,23 @@ static	void	set_realname _((char *));
 static	IrcVariable irc_variable[] =
 {
 	{ "ALWAYS_SPLIT_BIGGEST",	BOOL_TYPE_VAR,	DEFAULT_ALWAYS_SPLIT_BIGGEST, NULL, NULL, 0, 0 },
+	{ "AUTO_ACCEPT_DCC_CHAT",	BOOL_TYPE_VAR,	DEFAULT_AUTO_ACCEPT_DCC_CHAT, NULL, NULL, 0, 0 },
+	{ "AUTO_ACCEPT_DCC_SEND",	BOOL_TYPE_VAR,	DEFAULT_AUTO_ACCEPT_DCC_SEND, NULL, NULL, 0, 0 },
 	{ "AUTO_NSLOOKUP",		BOOL_TYPE_VAR,	DEFAULT_AUTO_NSLOOKUP, NULL, NULL, 0, 0 },
 	{ "AUTO_REJOIN",		INT_TYPE_VAR,	DEFAULT_AUTO_REJOIN, NULL, NULL, 0, 0 },
+	{ "AUTO_REOPER",		BOOL_TYPE_VAR,	DEFAULT_AUTO_REOPER, NULL, set_auto_reoper, 0, 0 },
 	{ "AUTO_UNMARK_AWAY",		BOOL_TYPE_VAR,	DEFAULT_AUTO_UNMARK_AWAY, NULL, NULL, 0, 0 },
 	{ "AUTO_WHOWAS",		BOOL_TYPE_VAR,	DEFAULT_AUTO_WHOWAS, NULL, NULL, 0, 0 },
 	{ "AUTOOP_DELAY",		INT_TYPE_VAR,	DEFAULT_AUTOOP_DELAY, NULL, NULL, 0, 0},
 	{ "AUTOOP_ENABLE",		BOOL_TYPE_VAR,	DEFAULT_AUTOOP_ENABLE, NULL, NULL, 0, 0},
 	{ "AWAY_RECORDER",		BOOL_TYPE_VAR,	DEFAULT_AWAY_RECORDER, NULL, NULL, 0, 0},
 	{ "BAN_TIMEOUT",		INT_TYPE_VAR,	DEFAULT_BAN_TIMEOUT, NULL, NULL, 0, 0},
+	{ "BANNER",			STR_TYPE_VAR,	0, NULL, NULL, 0, 0},
 	{ "BEEP",			BOOL_TYPE_VAR,	DEFAULT_BEEP, NULL, NULL, 0, 0 },
 	{ "BEEP_MAX",			INT_TYPE_VAR,	DEFAULT_BEEP_MAX, NULL, NULL, 0, 0 },
 	{ "BEEP_ON_MSG",		STR_TYPE_VAR,	0, NULL, set_beep_on_msg, 0, 0 },
 	{ "BEEP_WHEN_AWAY",		INT_TYPE_VAR,	DEFAULT_BEEP_WHEN_AWAY, NULL, NULL, 0, 0 },
+	{ "BLINK_VIDEO",		BOOL_TYPE_VAR,	DEFAULT_BLINK_VIDEO, NULL, NULL, 0, 0 },
 	{ "BOLD_VIDEO",			BOOL_TYPE_VAR,	DEFAULT_BOLD_VIDEO, NULL, NULL, 0, 0 },
 	{ "CHANNEL_NAME_WIDTH",		INT_TYPE_VAR,	DEFAULT_CHANNEL_NAME_WIDTH, NULL, update_all_status, 0, 0 },
 	{ "CLIENT_INFORMATION",		STR_TYPE_VAR,	0, NULL, NULL, 0, 0 },
@@ -150,7 +159,7 @@ static	IrcVariable irc_variable[] =
 	{ "MSGLOG",			BOOL_TYPE_VAR,	0, NULL, msglogger, 0, 0 },
 	{ "MSGLOGFILE",			STR_TYPE_VAR,	0, NULL, NULL, 0, 0 },
 	{ "NO_CTCP_FLOOD",		BOOL_TYPE_VAR,	DEFAULT_NO_CTCP_FLOOD, NULL, NULL, 0, 0 },
-	{ "NO_LOG",			STR_TYPE_VAR,	0, NULL, NULL, 0, 0 },
+	{ "NO_LOG",			STR_TYPE_VAR,	0, NULL, set_nolog, 0, 0 },
 	{ "NOTIFY_HANDLER",		STR_TYPE_VAR, 	0, 0, set_notify_handler, 0, 0 },
 	{ "NOTIFY_LEVEL",		STR_TYPE_VAR,	0, NULL, set_notify_level, 0, 0 },
 	{ "NOTIFY_ON_TERMINATION",	BOOL_TYPE_VAR,	DEFAULT_NOTIFY_ON_TERMINATION, NULL, NULL, 0, VF_NODAEMON },
@@ -158,10 +167,17 @@ static	IrcVariable irc_variable[] =
 	{ "NUM_BANMODES",		INT_TYPE_VAR,   4, NULL, NULL, 0, 0 },
 	{ "NUM_OPMODES",		INT_TYPE_VAR,   4, NULL, NULL, 0, 0 },
 	{ "OLD_WHOIS_FORMAT",		BOOL_TYPE_VAR,	DEFAULT_OLD_WHOIS_FORMAT, NULL, NULL, 0, 0 },
+	{ "OPS_METHOD",			STR_TYPE_VAR,	0, NULL, set_ops_method_var, 0, 0 },
 	{ "PING_TYPE",			INT_TYPE_VAR,   DEFAULT_PING_TYPE, NULL, NULL, 0, 0 },
 	{ "PUB_LOGSTR",			STR_TYPE_VAR,   0, NULL, NULL, 0, 0 },
 	{ "REALNAME",			STR_TYPE_VAR,	0, 0, set_realname, 0, VF_NODAEMON },
 	{ "REASON_TYPE",		INT_TYPE_VAR,   DEFAULT_REASON_TYPE, NULL, NULL, 0, 0 },
+	{ "REJOIN_BADKEY",		BOOL_TYPE_VAR,	DEFAULT_REJOIN_BADKEY, NULL, NULL, 0, 0 },
+	{ "REJOIN_BANNED",		BOOL_TYPE_VAR,	DEFAULT_REJOIN_BANNED, NULL, NULL, 0, 0 },
+	{ "REJOIN_FULL",		BOOL_TYPE_VAR,	DEFAULT_REJOIN_FULL, NULL, NULL, 0, 0 },
+	{ "REJOIN_INTERVAL",		INT_TYPE_VAR,	DEFAULT_REJOIN_INTERVAL, NULL, NULL, 0, 0 },
+	{ "REJOIN_INVITEONLY",		BOOL_TYPE_VAR,	DEFAULT_REJOIN_INVITEONLY, NULL, NULL, 0, 0 },
+	{ "REJOIN_UNAVAILABLE",		BOOL_TYPE_VAR,	DEFAULT_REJOIN_UNAVAILABLE, NULL, NULL, 0, 0 },
 	{ "SCREEN_OPTIONS", 		STR_TYPE_VAR,	0, NULL, NULL, 0, VF_NODAEMON },
 	{ "SCROLL",			BOOL_TYPE_VAR,	DEFAULT_SCROLL, NULL, set_scroll, 0, 0 },
 	{ "SCROLL_LINES",		INT_TYPE_VAR,	DEFAULT_SCROLL_LINES, NULL, set_scroll_lines, 0, 0 },
@@ -224,6 +240,7 @@ static	IrcVariable irc_variable[] =
 	{ "USERHOST_NOTIFY",		BOOL_TYPE_VAR,	DEFAULT_USERHOST_NOTIFY, NULL, NULL, 0, 0 },
 	{ "VERBOSE_CTCP",		BOOL_TYPE_VAR,	DEFAULT_VERBOSE_CTCP, NULL, NULL, 0, 0 },
 	{ "VERBOSE_WHOKILL",		BOOL_TYPE_VAR,	DEFAULT_VERBOSE_WHOKILL, NULL, NULL, 0, 0 },
+	{ "VHOST",			STR_TYPE_VAR,	0, NULL, NULL, 0, 0 },
 	{ "WARN_OF_IGNORES",		BOOL_TYPE_VAR,	DEFAULT_WARN_OF_IGNORES, NULL, NULL, 0, 0 },
 	{ "XTERM_OPTIONS", 		STR_TYPE_VAR,	0, NULL, NULL, 0, VF_NODAEMON },
 	{ (char *) 0, 0, 0, 0, 0, 0, 0 }
@@ -301,6 +318,9 @@ init_variables()
 	set_notify_level(irc_variable[NOTIFY_LEVEL_VAR].string);
 	set_string_var(TIMESTAMP_VAR, DEFAULT_TIMESTAMP);
 	set_string_var(IGN_PARAM_VAR, DEFAULT_IGN_PARAM);
+	set_string_var(OPS_METHOD_VAR, DEFAULT_OPS_METHOD);
+	set_ops_method_var(irc_variable[OPS_METHOD_VAR].string);
+	set_string_var(BANNER_VAR, DEFAULT_BANNER);
 }
 
 /*
@@ -768,3 +788,53 @@ set_realname(value)
 {
 	strmcpy(realname, value, REALNAME_LEN);
 }
+
+static	void
+set_auto_reoper(value)
+	int	value;
+{
+	if (value == OFF)
+	{
+		clear_all_server_operator_pwd();
+	}
+}
+
+static	void
+set_ops_method_var(value)
+	char	*value;
+{
+	int len;
+
+	if (value)
+	{
+		len = strlen(value);
+	}
+
+	if (!value || !my_strnicmp(value, "DEFAULT", len))
+	{
+		set_ops_method(OPS_METHOD_DEFAULT);
+		set_string_var(OPS_METHOD_VAR, "DEFAULT");
+	}
+	else if (!my_strnicmp(value, "HYBRID", len))
+	{
+		set_ops_method(OPS_METHOD_HYBRID);
+		set_string_var(OPS_METHOD_VAR, "HYBRID");
+	}
+	else if (!my_strnicmp(value, "WALLCHOPS", len))
+	{
+		set_ops_method(OPS_METHOD_WALLCHOPS);
+		set_string_var(OPS_METHOD_VAR, "WALLCHOPS");
+	}
+	else if (!my_strnicmp(value, "AUTO", len))
+	{
+		set_ops_method(OPS_METHOD_AUTO);
+		set_string_var(OPS_METHOD_VAR, "AUTO");
+	}
+	else
+	{
+		say("Unknown /OPS method: %s", value);
+		set_ops_method(OPS_METHOD_DEFAULT);
+		set_string_var(OPS_METHOD_VAR, "DEFAULT");
+	}
+}
+
