@@ -3,15 +3,15 @@
  * about a nickname using WHOIS.... all the time hiding this from the user.  
  *
  *
- * Written By Michael Sandrof
+ * Written By Timothy Jensen
  *
- * Copyright(c) 1990 
+ * Copyright (c) 1999-2001 
  *
- * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
+ * See the COPYRIGHT file, or do a /HELP COPYRIGHT 
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: whois.c,v 1.3 1999/08/06 07:07:17 toast Exp $";
+static	char	rcsid[] = "@(#)$Id: whois.c,v 1.12 2001/12/15 02:01:06 toast Exp $";
 #endif
 
 #undef MONITOR_Q /* this one is for monitoring of the 'whois queue' (debug) */
@@ -34,6 +34,7 @@ static	char	rcsid[] = "@(#)$Id: whois.c,v 1.3 1999/08/06 07:07:17 toast Exp $";
 #include "ctcp.h"
 #include "comstud.h"
 #include "toast.h"
+#include "format.h"
 
 char	whois_nick[] = "#WHOIS#";
 char	wait_nick[] = "#WAIT#";
@@ -113,6 +114,21 @@ remove_from_whois_queue()
 	set_server_qhead(from_server, new->next);
 	if (new->next == (WhoisQueue *) 0)
 		set_server_qtail(from_server, (WhoisQueue *) 0);
+
+#ifdef MONITOR_Q
+	{
+		char	*type;
+		if (new->type == WHOIS_WHOIS)
+			type = "WHOIS";
+		else if (new->type == WHOIS_ISON)
+			type = "ISON";
+		else if (new->type == WHOIS_USERHOST)
+			type = "USERHOST";
+		else
+			type = "<unknown>";
+		put_it("--- %s %s", type, new->nick);
+	}
+#endif
 	return (new);
 }
 
@@ -205,6 +221,7 @@ userhost_returned(from, ArgList)
 		isuser,
 		parsed;
 	char	*queue_nicks;
+	char	*banner;
 
 	if (!ArgList[0])
 		return;
@@ -294,16 +311,32 @@ userhost_returned(from, ArgList)
 			if (do_hook(current_numeric, "%s %s %s %s %s", nick,
 			    isoper ? "+" : "-", ishere ? "+" : "-", user, host))
 			{
-				put_it("%s %s is %s@%s%s%s", numeric_banner(),
-					nick, user, host, isoper ?
-					" (Is an IRC operator)" : empty_string,
-					ishere ? empty_string : " (away)");
+				if (num_format_exists(-current_numeric))
+				{
+					put_it("%s",
+						numericformat(-current_numeric,
+						from, numeric_banner(),
+						ArgList));
+				}
+				else
+				{
+					put_it("%s%s is %s@%s%s%s",
+						numeric_banner(),
+						nick, user, host, isoper ?
+						" (Is an IRC operator)" :
+						empty_string,
+						ishere ? empty_string :
+						" (away)");
+				}
 				if (host && isdigit(*(host+strlen(host)-1)) &&
 					get_int_var(AUTO_NSLOOKUP_VAR))
 				{
 					char *nsl;
 					nsl = do_nslookup(host);
-					put_it("*** %s is %s", host, nsl ? nsl : "unknown.");
+					banner = get_string_var(BANNER_VAR);
+					put_it("%s%s is %s",
+						banner ? banner : empty_string,
+						host, nsl ? nsl : "unknown.");
 				}
 			}
 		}
@@ -340,6 +373,7 @@ whois_name(from, ArgList)
 		*ptr,
 		*name;
 	WhoisStuff *whois_stuff;
+	char	*banner;
 
 	PasteArgs(ArgList, 4);
 	nick = ArgList[0];
@@ -375,23 +409,46 @@ whois_name(from, ArgList)
 		if (do_hook(current_numeric, "%s %s %s %s %s %s", from, nick,
 				user, host, channel, name))
 		{
-			if (!get_int_var(OLD_WHOIS_FORMAT_VAR)) {
-			  put_it("\002[\002 Whois \002%s\002!%s@%s \002]\002", nick, user, host);
-			  message_from((char *) 0, LOG_CRAP);
-			  put_it(":\037 ircname  \037: %s", name);
-			} else
-			  put_it("%s %s is %s@%s (%s)", numeric_banner(), nick,
-					user, host, name);
+			if (!get_int_var(OLD_WHOIS_FORMAT_VAR))
+			{
+				put_it("%s", parseformat(WHOIS_USERHOST_FMT,
+					nick, user, host));
+				put_it("%s", parseformat(WHOIS_IRCNAME_FMT,
+					nick, name));
+				message_from((char *) 0, LOG_CRAP);
+			}
+			else if (num_format_exists(-current_numeric))
+			{
+				put_it("%s",
+					numericformat(-current_numeric,
+					from, numeric_banner(),
+					ArgList));
+			}
+			else
+			{
+				put_it("%s%s is %s@%s (%s)",
+					numeric_banner(), nick, user,
+					host, name);
+			}
 			if (host && isdigit(*(host+strlen(host)-1)) &&
 				get_int_var(AUTO_NSLOOKUP_VAR))
 			{
 				char *nsl;
 				nsl = do_nslookup(host);
 				if (!get_int_var(OLD_WHOIS_FORMAT_VAR))
-				  put_it(":\037 nslookup \037: %s -> %s", host,
-					nsl ? nsl : "unknown.");
+				{
+					put_it("%s",
+						parseformat(WHOIS_NSLOOKUP_FMT,
+						nick, host,
+						nsl ? nsl : "unknown"));
+				}
 				else
-				  put_it("*** %s is %s", host, nsl ? nsl : "unknown.");
+				{
+					banner = get_string_var(BANNER_VAR);
+					put_it("%s%s is %s",
+						banner ? banner : empty_string,
+						host, nsl ? nsl : "unknown.");
+				}
 			} 
 		}
 	}
@@ -446,16 +503,30 @@ whowas_name(from, ArgList)
 	{
 		ignore_whois_crap = 0;
 		if (do_hook(current_numeric, "%s %s %s %s %s %s", from, nick,
-				user, host, channel, name))
-			if (get_int_var(OLD_WHOIS_FORMAT_VAR))
-			  put_it("%s %s was %s@%s (%s) on channel %s",
-				numeric_banner(), nick, user, host, name,
-				(*channel == '*') ? "*private*" : channel);
-			else {
-			  put_it("\002[\002 Whowas \002%s\002!%s@%s \002]\002",
-				nick, user, host);
-			  put_it(":\037 ircname  \037: %s", name);
+			user, host, channel, name))
+		{
+			if (!get_int_var(OLD_WHOIS_FORMAT_VAR))
+			{
+				put_it("%s", parseformat(WHOWAS_USERHOST_FMT,
+					nick, user, host));
+				put_it("%s", parseformat(WHOIS_IRCNAME_FMT,
+					nick, name));
 			}
+			else if (num_format_exists(-current_numeric))
+			{
+				put_it("%s",
+					numericformat(-current_numeric,
+					from, numeric_banner(),
+					ArgList));
+			}
+			else
+			{
+				put_it("%s%s was %s@%s (%s) on channel %s",
+					numeric_banner(), nick, user,
+					host, name, (*channel == '*') ?
+					"*private*" : channel);
+			}
+		}
 	}
 	set_lastlog_msg_level(lastlog_level);
 }
@@ -485,10 +556,23 @@ whois_channels(from, ArgList)
 	else
 	{
 		if (do_hook(current_numeric, "%s %s", from, line))
+		{
 			if (!get_int_var(OLD_WHOIS_FORMAT_VAR))
-			  put_it(":\037 channels \037: %s", line);
+			{
+				put_it("%s", parseformat(WHOIS_CHANNELS_FMT,
+					ArgList[0], line));
+			}
+			else if (num_format_exists(-current_numeric))
+			{
+				put_it("%s", numericformat(-current_numeric,
+					from, numeric_banner(), ArgList));
+			}
 			else
-			  put_it("%s on channels: %s", numeric_banner(), line);
+			{
+				put_it("%son channels: %s", numeric_banner(),
+					line);
+			}
+		}
 	}
 }
 
@@ -507,7 +591,8 @@ whois_server(from, ArgList)
 	char	*from;
 	char	**ArgList;
 {
-	char	*server,
+	char	*nick,
+		*server,
 		*ptr;
 	char	*line;
 	WhoisStuff *whois_stuff;
@@ -517,11 +602,13 @@ whois_server(from, ArgList)
 		return;
 	if (ArgList[2])
 	{
+		nick = ArgList[0];
 		server = ArgList[1];
 		line = ArgList[2];
 	}
 	else
 	{
+		nick = empty_string;
 		server = ArgList[0];
 		line = ArgList[1];
 	}
@@ -538,11 +625,25 @@ whois_server(from, ArgList)
 	{
 		message_from((char *) 0, LOG_CRAP);
 		if (do_hook(current_numeric, "%s %s %s", from, server, line))
+		{
 			if (!get_int_var(OLD_WHOIS_FORMAT_VAR))
-			  put_it(":\037 server   \037: %s (%s)", server, line);
+			{
+				put_it("%s", parseformat(WHOIS_SERVER_FMT,
+					nick, server, line));
+			}
+			else if (num_format_exists(-current_numeric))
+			{
+				put_it("%s",
+					numericformat(-current_numeric,
+					from, numeric_banner(),
+					ArgList));
+			}
 			else
-			  put_it("%s on irc via server %s (%s)",
-				numeric_banner(), server, line);
+			{
+				put_it("%son irc via server %s (%s)",
+					numeric_banner(), server, line);
+			}
+		}
 	}
 }
 
@@ -569,14 +670,28 @@ whois_oper(from, ArgList)
 		if ((nick = ArgList[0]) != NULL)
 		{
 			if (do_hook(current_numeric, "%s %s %s", from, nick,
-					ArgList[1]))
+				ArgList[1]))
+			{
 				if (!get_int_var(OLD_WHOIS_FORMAT_VAR))
-				  put_it(":\037 operator \037: %s %s", nick, ArgList[1]);
+				{
+					put_it("%s",
+						parseformat(WHOIS_OPER_FMT,
+						nick, ArgList[1]));
+				}
+				else if (num_format_exists(-current_numeric))
+				{
+					put_it("%s",
+						numericformat(-current_numeric,
+						from, numeric_banner(),
+						ArgList));
+				}
 				else
-				  put_it("%s %s %s%s", numeric_banner(), nick,
-					ArgList[1], (server_list[from_server].version >
-					Server2_7) ? empty_string
-						   : " (is an IRC operator)");
+				{
+					put_it("%s%s %s%s", numeric_banner(),
+						nick, ArgList[1],
+						(server_list[from_server].version > Server2_7) ? empty_string : " (is an IRC operator)");
+				}
+			}
 		}
 	}
 }
@@ -605,14 +720,28 @@ whois_admin(from, ArgList)
 		if ((nick = ArgList[0]) != NULL)
 		{
 			if (do_hook(current_numeric, "%s %s %s", from, nick,
-					ArgList[1]))
+				ArgList[1]))
+			{
 				if (!get_int_var(OLD_WHOIS_FORMAT_VAR))
-				  put_it(":\037 admin    \037: %s %s", nick, ArgList[1]);
+				{
+					put_it("%s",
+						parseformat(WHOIS_ADMIN_FMT,
+						nick, ArgList[1]));
+				}
+				else if (num_format_exists(-current_numeric))
+				{
+					put_it("%s",
+						numericformat(-current_numeric,
+						from, numeric_banner(),
+						ArgList));
+				}
 				else
-				  put_it("%s %s %s%s", numeric_banner(), nick,
-					ArgList[1], (server_list[from_server].version >
-					Server2_7) ? empty_string
-						   : " (is an IRC admin)");
+				{
+					put_it("%s%s %s%s", numeric_banner(),
+						nick, ArgList[1],
+						(server_list[from_server].version > Server2_7) ? empty_string : " (is an IRC admin)");
+				}
+			}
 		}
 	}
 }
@@ -622,45 +751,46 @@ whois_lastcom(from, ArgList)
 	char	*from;
 	char	**ArgList;
 {
+	time_t	idle;
+
 	if (!ignore_whois_crap)
 	{
-/*		char	flag;*/
 		char	*nick, *idle_str, *signon_str;
-		int	idle;
 		time_t	signon;
 
 		PasteArgs(ArgList, 3);
 		message_from((char *) 0, LOG_CRAP);
-		if ((nick = ArgList[0]) && (idle_str = ArgList[1]) && (signon_str = ArgList[2]) &&
-				do_hook(current_numeric, "%s %s %s %s", from,
-					nick, idle_str, ArgList[2]))
+		if ((nick = ArgList[0]) && (idle_str = ArgList[1]) &&
+			(signon_str = ArgList[2]) &&
+			do_hook(current_numeric, "%s %s %s %s", from,
+			nick, idle_str, ArgList[2]))
 		{
-/*
-			if ((idle = atoi(idle_str)) > 59)
+			if (!get_int_var(OLD_WHOIS_FORMAT_VAR))
 			{
-				idle = idle/60;
-				flag = 1;
+				idle = atol(idle_str);
+				put_it("%s", parseformat(WHOIS_IDLE_FMT,
+					nick, idle_str, tdiff(idle)));
+			  	if ((signon = atol(signon_str)) != 0)
+				{
+			    		char tmp[128];
+			    		struct tm *time_val;
+			    		time_val = localtime(&signon);
+			    		strftime(tmp, 128, "%c %Z", time_val);
+					put_it("%s",
+						parseformat(WHOIS_SIGNON_FMT,
+						nick, signon_str, tmp));
+				}
+			}
+			else if (num_format_exists(-current_numeric))
+			{
+				put_it("%s", numericformat(-current_numeric,
+					from, numeric_banner(), ArgList));
 			}
 			else
-				flag = 0;
-*/
-
-			if (get_int_var(OLD_WHOIS_FORMAT_VAR))
-/*
-			  put_it("%s %s has been idle %d %ss", numeric_banner(),
-				nick, idle, flag? "minute": "second");
-*/
-			  put_it("%s %s has been idle %s", numeric_banner(),
-				nick, function_tdiff(idle_str));
-			else {
-			  put_it(":\037 idle set \037: %s", function_tdiff(idle_str));
-			  if ((signon = atol(signon_str)) != 0) {
-			    char tmp[128];
-			    struct tm *time_val;
-			    time_val = localtime(&signon);
-			    strftime(tmp, 128, "%c %Z", time_val);
-			    put_it(":\037 sign on  \037: %s", tmp);
-			  }
+			{
+				put_it("%s%s has been idle %s",
+					numeric_banner(), nick,
+					tdiff(atoi(idle_str)));
 			}
 		}
 	}
@@ -681,7 +811,9 @@ whois_chop(from, ArgList)
 	whois_stuff = get_server_whois_stuff(from_server);
 	PasteArgs(ArgList, 1);
 	if (ignore_whois_crap)
+	{
 		whois_stuff->chop = 1;
+	}
 	else
 	{
 		char	*nick;
@@ -689,12 +821,18 @@ whois_chop(from, ArgList)
 		if ((nick = ArgList[0]) != NULL)
 		{
 			if (do_hook(current_numeric, "%s %s %s",from, nick,
-					ArgList[1]))
+				ArgList[1]))
+			{
 				if (num_format_exists(-current_numeric))
+				{
 					put_it("%s", numericformat(-current_numeric, from, numeric_banner(), ArgList));
+				}
 				else
-					put_it("%s %s (is a channel operator)",
+				{
+					put_it("%s%s (is a channel operator)",
 						numeric_banner(), nick, ArgList[1]);
+				}
+			}
 		}
 	}
 }
@@ -755,16 +893,41 @@ no_such_nickname(from, ArgList)
 	char	*from,
 		**ArgList;
 {
-	char	*nick;
+	WhoisStuff *whois_stuff;
+	char	*nick, *ptr;
+
+	whois_stuff = get_server_whois_stuff(from_server);
 
 	PasteArgs(ArgList, 1);
 	nick = ArgList[0];
 	if (do_hook(current_numeric, "%s %s %s", from, nick, ArgList[1]))
+	{
 		if (num_format_exists(-current_numeric))
-			put_it("%s", numericformat(-current_numeric, from, numeric_banner(), ArgList));
+		{
+			put_it("%s", numericformat(-current_numeric,
+				from, numeric_banner(), ArgList));
+		}
 		else
-			put_it("%s %s: %s", numeric_banner(), nick, ArgList[1]);
+		{
+			put_it("%s%s: %s", numeric_banner(), nick, ArgList[1]);
+		}
+	}
+	remnick(nick);
 	notify_mark(nick, 0, 0);
+	ptr = whois_queue_head();
+	if (ptr && (my_stricmp(ptr, nick) == 0))
+	{
+		WhoisQueue *thing;
+
+		thing = remove_from_whois_queue();
+		whois_stuff->not_on = 0;
+		thing->func(whois_stuff, thing->nick, thing->text);
+		new_free(&whois_stuff->channels);
+		new_free(&thing->nick);
+		new_free(&thing->text);
+		new_free(&thing);
+		ignore_whois_crap = 0;
+	}
 	if ((get_server_version(from_server) > Server2_5) &&
 	    get_int_var(AUTO_WHOWAS_VAR))
 		/*
@@ -772,7 +935,6 @@ no_such_nickname(from, ArgList)
 		 * with multiple copies. (Toast, 14 Oct 1997)
 		 */
 		send_to_server("WHOWAS %s 1", nick);
-	remnick(nick);
 }
 
 /*
@@ -785,8 +947,6 @@ user_is_away(from, ArgList)
 	char	*from,
 		**ArgList;
 {
-	static	char	*last_away_msg = (char *) 0,
-			*last_away_nick = (char *) 0;
 	char	*message,
 		*who;
 	WhoisStuff *whois_stuff;
@@ -806,27 +966,31 @@ if (!from)
 		{
 			if (!show_away_flag && get_int_var(SHOW_AWAY_ONCE_VAR))
 			{
-/*
-				if (!last_away_msg || strcmp(last_away_nick,
-					from) || strcmp(last_away_msg, message))
-				{
-					malloc_strcpy(&last_away_nick, from);
-					malloc_strcpy(&last_away_msg, message);
-				}
-*/
 				if (!seen_away_msg(who, message))
 					add_to_away(who, message);
 				else return;
 			}
 			if (do_hook(current_numeric, "%s %s", who, message))
-				if (in_whois && !get_int_var(OLD_WHOIS_FORMAT_VAR))
-				  put_it(":\037 away set \037: %s", message);
+			{
+				if (in_whois &&
+					!get_int_var(OLD_WHOIS_FORMAT_VAR))
+				{
+					put_it("%s", parseformat(WHOIS_AWAY_FMT,
+						whois_stuff->nick, message));
+				}
+				else if (num_format_exists(-current_numeric))
+				{
+					put_it("%s",
+						numericformat(-current_numeric,
+						from, numeric_banner(),
+						ArgList));
+				}
 				else
-				  if (num_format_exists(-current_numeric))
-					put_it("%s", numericformat(-current_numeric, from, numeric_banner(), ArgList));
-				  else
-				  	put_it("%s %s is away: %s",numeric_banner(),
-						who, message);
+				{
+				  	put_it("%s%s is away: %s",
+						numeric_banner(), who, message);
+				}
+			}
 		}
 		eat_away = 0;
 	}
@@ -1114,6 +1278,8 @@ whois_new_wallops(stuff, nick, text)
 	int	flag,
 	level;
 	char	*high;
+	char	*tmp;
+	char	*ptr;
 
 	flag = is_ignored(nick, IGNORE_WALLOPS);
 	if (flag != IGNORED)
@@ -1122,37 +1288,49 @@ whois_new_wallops(stuff, nick, text)
 			high = &highlight_char;
 		else
 			high = empty_string;
-		if (stuff && (ignore_usernames & IGNORE_WALLOPS))
+		if (stuff)
 		{
-			char	*ptr;
 
 			ptr = (char *) new_malloc(strlen(stuff->user) +
 				strlen(stuff->host) + 2);
 			strcpy(ptr, stuff->user);
 			strcat(ptr, "@");
 			strcat(ptr, stuff->host);
-			if (is_ignored(ptr, IGNORE_WALLOPS) == IGNORED)
+			if ((ignore_usernames & IGNORE_WALLOPS) && is_ignored(ptr, IGNORE_WALLOPS) == IGNORED)
 			{
 				new_free(&ptr);
 				return;
 			}
-			new_free(&ptr);
 		}
 		message_from(nick, LOG_WALLOP);
 		level = set_lastlog_msg_level(LOG_WALLOP);
 		if (stuff)
 		{
 			if (do_hook(WALLOP_LIST, "%s %s %s", nick,
-					(stuff->oper ? "+" : "-"), text))
-				put_it("%s!%s%s!%s %s", high, nick,
-					stuff->oper ? "*" : empty_string,
-					high, text);
+				(stuff->oper ? "+" : "-"), text))
+			{
+
+				tmp = FromUserHost;
+				FromUserHost = ptr;
+				put_it("%s",
+					parseformat(WALLOPS_FMT, nick, text,
+					stuff->oper ? "*" : empty_string));
+				FromUserHost = tmp;
+			}
 		}
 		else
 		{
 			if (do_hook(WALLOP_LIST, "%s - %s", nick, text))
-				put_it("%s!%s!%s %s", high, nick, high, text);
+			{
+				tmp = FromUserHost;
+				FromUserHost = "unknown@unknown";
+				put_it("%s",
+					parseformat(WALLOPS_FMT, nick, text,
+					empty_string));
+				FromUserHost = tmp;
+			}
 		}
+		new_free(&ptr);
 		if (beep_on_level & LOG_WALLOP)
 			beep_em(1);
 		set_lastlog_msg_level(level);
@@ -1318,3 +1496,25 @@ userhost_cmd_returned(stuff, nick, text)
 	strcat(args, stuff->host ? stuff->host : empty_string);
 	parse_line((char *) 0, text, args, 0, 0);
 }
+
+extern	void
+show_whois_queue()
+{
+	WhoisQueue *tmp = get_server_qhead(from_server);
+	char *type;
+
+	while (tmp)
+	{
+		if (tmp->type == WHOIS_WHOIS)
+			type = "WHOIS";
+		else if (tmp->type == WHOIS_ISON)
+			type = "ISON";
+		else if (tmp->type == WHOIS_USERHOST)
+			type = "USERHOST";
+		else
+			type = "<unknown>";
+		say("[%s] [%s] %s %p", tmp->nick, tmp->text, type, tmp->func);
+
+		tmp = tmp->next;
+	}
+}
