@@ -10,7 +10,7 @@
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: window.c,v 1.1.1.1 1999/07/16 21:21:44 toast Exp $";
+static	char	rcsid[] = "@(#)$Id: window.c,v 1.3 2001/11/29 22:26:49 toast Exp $";
 #endif
 
 #include "irc.h"
@@ -1437,7 +1437,7 @@ is_bound(channel, server)
 	Window *tmp;
 	int	flag = 1;
 
-	while (tmp = traverse_all_windows(&flag))
+	while (NULL != (tmp = traverse_all_windows(&flag)))
 		if (tmp->server == server && tmp->bind_channel
 		&& !my_stricmp(tmp->bind_channel, channel))
 			return 1;
@@ -1452,7 +1452,7 @@ unbind_channel(channel, server)
 	Window *tmp;
 	int	flag = 1;
 
-	while (tmp = traverse_all_windows(&flag))
+	while (NULL != (tmp = traverse_all_windows(&flag)))
 		if (tmp->server == server && tmp->bind_channel
 		&& !my_stricmp(tmp->bind_channel, channel))
 		{
@@ -1468,7 +1468,7 @@ get_bound_channel(window)
 	Window *tmp;
 	int	flag = 1;
 
-	while (tmp = traverse_all_windows(&flag))
+	while (NULL != (tmp = traverse_all_windows(&flag)))
 		if (tmp == window)
 			return tmp->bind_channel;
 	return empty_string;
@@ -1478,8 +1478,9 @@ get_bound_channel(window)
  * get_window_server: returns the server index for the window with the given
  * refnum 
  */
-int	get_window_server(refnum)
-unsigned int	refnum;
+int
+get_window_server(refnum)
+	unsigned int	refnum;
 {
 	Window	*tmp;
 
@@ -1770,7 +1771,7 @@ list_a_window(window, len)
  * by displaying their refnums, current channel, and current nick 
  */
 void
-list_windows()
+list_windows(void)
 {
 	Window	*tmp;
 	int	flag = 1;
